/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Workflow.Activities;
using Composite.C1Console.Events;
using Composite.Data;
using Composite.Core.Types;
using Composite.C1Console.Workflow;
using Composite.C1Console.Actions;
using Composite.Data.GeneratedTypes;
using Composite.Data.DynamicTypes;
using Composite.C1Console.Users;
using Composite.Core.Logging;
using Composite.Data.Validation.ClientValidationRules;
using Composite.Core.ResourceSystem;
using Composite.Data.Types;


namespace Composite.C1Console.Elements.ElementProviderHelpers.AssociatedDataElementProviderHelper
{
    [AllowPersistingWorkflow(WorkflowPersistingType.Idle)]
    public sealed partial class AddDataFolderExWorkflow : Composite.C1Console.Workflow.Activities.FormsWorkflow
    {
        private string TypesBindingName { get { return "Types"; } }
        private string SelectedTypeBindingName { get { return "SelectedType"; } }

        private string NewTypeNameBindingName { get { return "NewTypeName"; } }
        private string NewTypeNamespaceBindingName { get { return "NewTypeNamespace"; } }
        private string NewTypeTitleBindingName { get { return "NewTypeTitle"; } }
        private string DataFieldDescriptorsBindingName { get { return "DataFieldDescriptors"; } }
        private string LabelFieldNameBindingName { get { return "LabelFieldName"; } }
        private string HasRecycleBinBindingName { get { return "HasRecycleBin"; } }
        private string HasPublishingBindingName { get { return "HasPublishing"; } }
        private string HasLocalizationBindingName { get { return "HasLocalization"; } }


        public AddDataFolderExWorkflow()
        {
            InitializeComponent();
        }



        private void ShouldCreateNewType(object sender, ConditionalEventArgs e)
        {
            e.Result = this.ExtraPayload == "CreateNew";
        }



        private void UnunsedTypesExist(object sender, ConditionalEventArgs e)
        {
            Type associatedType = TypeManager.GetType(this.Payload);

            IEnumerable<Type> accociationTypes = PageFolderFacade.GetAllFolderTypes();
            IEnumerable<Type> usedAccociationTypes = PageFolderFacade.GetDefinedFolderTypes(this.GetDataItemFromEntityToken<IPage>());

            e.Result = accociationTypes.Except(usedAccociationTypes).Any();
        }



        private void UseExistingType(object sender, ConditionalEventArgs e)
        {
            Type type = this.GetBinding<Type>(this.SelectedTypeBindingName);

            e.Result = type != typeof(AddDataFolderExWorkflow);
        }



        private void IsTypeCreated(object sender, ConditionalEventArgs e)
        {
            e.Result = this.BindingExist(this.SelectedTypeBindingName);
        }        



        private void selectTypeCodeActivity_Initialize_ExecuteCode(object sender, EventArgs e)
        {
            Type associatedType = TypeManager.GetType(this.Payload);

            IEnumerable<Type> accociationTypes = PageFolderFacade.GetAllFolderTypes();
            IEnumerable<Type> usedAccociationTypes = PageFolderFacade.GetDefinedFolderTypes(this.GetDataItemFromEntityToken<IPage>());

            Dictionary<Type, string> types = new Dictionary<Type, string>();
            foreach (var kvp in accociationTypes.Except(usedAccociationTypes).ToDictionary(f => f, f => f.GetTypeTitle()))
            {
                types.Add(kvp.Key, kvp.Value);
            }

            this.Bindings.Add(this.TypesBindingName, types);
            this.Bindings.Add(this.SelectedTypeBindingName, types.Keys.First());
        }



        private void selectTypeCodeActivity_StartCreateNewTypeWorkflow_ExecuteCode(object sender, EventArgs e)
        {
            this.CloseCurrentView();
            this.ExecuteAction(this.EntityToken, new WorkflowActionToken(typeof(AddDataFolderExWorkflow), this.ActionToken.PermissionTypes) { Payload = this.Payload, ExtraPayload = "CreateNew", DoIgnoreEntityTokenLocking = this.ActionToken.IgnoreEntityTokenLocking });
        }



        private void createNewTypeCodeActivity_Initialize_ExecuteCode(object sender, EventArgs e)
        {
            if (this.BindingExist(this.NewTypeNameBindingName) == false)
            {
                this.Bindings.Add(this.NewTypeNameBindingName, "");
                this.Bindings.Add(this.NewTypeNamespaceBindingName, UserSettings.LastSpecifiedNamespace);
                this.Bindings.Add(this.NewTypeTitleBindingName, "");
                this.Bindings.Add(this.DataFieldDescriptorsBindingName, new List<DataFieldDescriptor>());
                this.Bindings.Add(this.LabelFieldNameBindingName, "");

                this.Bindings.Add(this.HasRecycleBinBindingName, false);
                this.Bindings.Add(this.HasPublishingBindingName, false);
                this.Bindings.Add(this.HasLocalizationBindingName, false);

                this.BindingsValidationRules.Add(this.NewTypeNameBindingName, new List<ClientValidationRule> { new NotNullClientValidationRule() });
                this.BindingsValidationRules.Add(this.NewTypeNamespaceBindingName, new List<ClientValidationRule> { new NotNullClientValidationRule() });
                this.BindingsValidationRules.Add(this.NewTypeTitleBindingName, new List<ClientValidationRule> { new NotNullClientValidationRule() });
            }

            //if (RuntimeInformation.IsDebugBuild)
            //{
            //    DynamicTempTypeCreator dynamicTempTypeCreator = new DynamicTempTypeCreator("IAggTest");

            //    this.UpdateBinding(this.NewTypeNameBindingName, dynamicTempTypeCreator.TypeName);
            //    this.UpdateBinding(this.NewTypeTitleBindingName, dynamicTempTypeCreator.TypeTitle);
            //    this.UpdateBinding(this.DataFieldDescriptorsBindingName, dynamicTempTypeCreator.DataFieldDescriptors);
            //}
        }        



        private void saceNewTypeCodeActivity_Save_ExecuteCode(object sender, EventArgs e)
        {
            bool saveResult = false;

            try
            {
                string typeName = this.GetBinding<string>(this.NewTypeNameBindingName);
                string typeNamespace = this.GetBinding<string>(this.NewTypeNamespaceBindingName);
                string typeTitle = this.GetBinding<string>(this.NewTypeTitleBindingName);
                bool hasRecycleBin = this.GetBinding<bool>(this.HasRecycleBinBindingName);
                bool hasPublishing = this.GetBinding<bool>(this.HasPublishingBindingName);
                bool hasLocalization = this.GetBinding<bool>(this.HasLocalizationBindingName);
                string labelFieldName = this.GetBinding<string>(this.LabelFieldNameBindingName);
                List<DataFieldDescriptor> dataFieldDescriptors = this.GetBinding<List<DataFieldDescriptor>>(this.DataFieldDescriptorsBindingName);

                GeneratedTypesHelper helper = new GeneratedTypesHelper();
                helper = new GeneratedTypesHelper();

                string errorMessage;
                if (helper.ValidateNewTypeName(typeName, out errorMessage) == false)
                {
                    this.ShowFieldMessage("NewTypeName", errorMessage);
                    SetSaveStatus(false);
                    return;
                }

                if (helper.ValidateNewTypeNamespace(typeNamespace, out errorMessage) == false)
                {
                    this.ShowFieldMessage("NewTypeNamespace", errorMessage);
                    SetSaveStatus(false);
                    return;
                }

                if (helper.ValidateNewTypeFullName(typeName, typeNamespace, out errorMessage) == false)
                {
                    this.ShowFieldMessage("NewTypeName", errorMessage);
                    SetSaveStatus(false);
                    return;
                }

                if (helper.ValidateNewFieldDescriptors(dataFieldDescriptors, out errorMessage) == false)
                {
                    this.ShowMessage(
                            DialogType.Warning,
                            "${Composite.Management, AssociatedDataElementProviderHelper.AddDataFolderExCreateNewType.ErrorTitle}",
                            errorMessage
                        );
                    SetSaveStatus(false);
                    return;
                }

                if (helper.IsEditProcessControlledAllowed)
                {
                    helper.SetPublishControlled(hasPublishing);
                    helper.SetLocalizedControlled(hasLocalization);
                }

                helper.SetNewTypeFullName(typeName, typeNamespace);
                helper.SetNewTypeTitle(typeTitle);
                helper.SetNewFieldDescriptors(dataFieldDescriptors, labelFieldName);

                Type targetType = TypeManager.GetType(this.Payload);
                helper.SetForeignKeyReference(targetType, Composite.Data.DataAssociationType.Aggregation);

                helper.CreateType(false);

                this.UpdateBinding(this.SelectedTypeBindingName, helper.InterfaceType);

                UserSettings.LastSpecifiedNamespace = typeNamespace;

                saveResult = true;
            }
            catch (Exception ex)
            {
                LoggingService.LogCritical("AddNewAggregationTypeWorkflow", ex);

                this.ShowMessage(DialogType.Error, ex.Message, ex.Message);                
            }

            SetSaveStatus(saveResult);
        }



        private void finalizeCodeActivity_Finalize_ExecuteCode(object sender, EventArgs e)
        {
            DataEntityToken dataEntityToken = (DataEntityToken)this.EntityToken;
            Type type = this.GetBinding<Type>(SelectedTypeBindingName);

            IPage page = (IPage)dataEntityToken.Data;

            page.AddFolderDefinition(type.GetImmutableTypeId());

            SpecificTreeRefresher specificTreeRefresher = this.CreateSpecificTreeRefresher();
            specificTreeRefresher.PostRefreshMesseges(this.EntityToken);
        }



        private void noTypesToAddCodeActivity_ShowMessage_ExecuteCode(object sender, EventArgs e)
        {
            Type associatedType = TypeManager.GetType(this.Payload);
            IEnumerable<Type> accociationTypes = PageFolderFacade.GetAllFolderTypes();

            if (accociationTypes.Any() == false)
            {
                this.ShowMessage(
                    DialogType.Message,
                    "${Composite.Management, AssociatedDataElementProviderHelper.AddDataFolderExWorkflow.NoTypesTitle}",
                    "${Composite.Management, AssociatedDataElementProviderHelper.AddDataFolderExWorkflow.NoTypesMessage}"
                    );
            }
            else
            {
                this.ShowMessage(
                    DialogType.Message,
                    "${Composite.Management, AssociatedDataElementProviderHelper.AddDataFolderExWorkflow.NoUnusedTypesTitle}",
                    "${Composite.Management, AssociatedDataElementProviderHelper.AddDataFolderExWorkflow.NoUnusedTypesMessage}"
                    );

            }
        }                
    }
}
