/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Transactions;
using System.Workflow.Activities;
using Composite.C1Console.Actions;
using Composite.C1Console.Events;
using Composite.C1Console.Workflow;
using Composite.Core.ResourceSystem;
using Composite.Data;
using Composite.Data.DynamicTypes;
using Composite.Data.GeneratedTypes;
using Composite.Data.Transactions;
using Composite.Data.Types;


namespace Composite.C1Console.Elements.ElementProviderHelpers.AssociatedDataElementProviderHelper
{
    [AllowPersistingWorkflow(WorkflowPersistingType.Idle)]
    public sealed partial class AddMetaDataWorkflow : Composite.C1Console.Workflow.Activities.FormsWorkflow
    {
        private static string TypesBindingName { get { return "Types"; } }
        private static string SelectedTypeBindingName { get { return "SelectedType"; } }
        private static string FieldGroupNameBindingName { get { return "FieldGroupName"; } }
        private static string FieldGroupLabelBindingName { get { return "FieldGroupLabel"; } }
        private static string ContainersBindingName { get { return "Containers"; } }
        private static string SelectedContainerBindingName { get { return "SelectedContainer"; } }
        private static string StartDisplayOptionsBindingName { get { return "StartDisplayOptions"; } }
        private static string SelectedStartDisplayBindingName { get { return "SelectedStartDisplay"; } }
        private static string InheritDisplayOptionsBindingName { get { return "InheritDisplayOptions"; } }
        private static string SelectedInheritDisplayBindingName { get { return "SelectedInheritDisplay"; } }

        private static string DataAssociationVisabilityDescriptionBindingName { get { return "DataAssociationVisabilityRule"; } }



        public AddMetaDataWorkflow()
        {
            InitializeComponent();
        }



        private IEnumerable<Type> GetAddebleTypes()
        {
            IEnumerable<Type> accociationTypes = PageMetaDataFacade.GetAllMetaDataTypes();

            return accociationTypes;
        }



        private IPage GetCurrentPage()
        {
            if ((this.EntityToken is DataEntityToken))
            {
                return this.GetDataItemFromEntityToken<IPage>();
            }
            else
            {
                return null;
            }
        }



        private Guid GetCurrentPageId()
        {
            IPage page = GetCurrentPage();

            if (page != null)
            {
                return page.Id;
            }
            else
            {
                return Guid.Empty;
            }
        }



        private bool IsAnyPagesAffected()
        {
            PageMetaDataDescription dataAssociationVisabilityRule = this.GetBinding<PageMetaDataDescription>(DataAssociationVisabilityDescriptionBindingName);

            IPage page = GetCurrentPage();

            return page.GetMetaDataAffectedPages(dataAssociationVisabilityRule.StartLevel, dataAssociationVisabilityRule.Levels).Any();
        }




        private void DoesTypesToAddExists(object sender, ConditionalEventArgs e)
        {
            e.Result = GetAddebleTypes().FirstOrDefault() != null;
        }



        private void DidFieldNameValidate(object sender, ConditionalEventArgs e)
        {
            PageMetaDataDescription dataAssociationVisabilityRule = this.GetBinding<PageMetaDataDescription>(DataAssociationVisabilityDescriptionBindingName);

            e.Result = dataAssociationVisabilityRule != null;
        }

        

        private void DoesTargetDataExists(object sender, ConditionalEventArgs e)
        {
            e.Result = IsAnyPagesAffected();
        }



        private void selectTypeCodeActivity_Initialize_ExecuteCode(object sender, EventArgs e)
        {
            if (this.BindingExist(TypesBindingName) == false)
            {
                Dictionary<Type, string> types = GetAddebleTypes().OrderBy(f => f.GetTypeTitle()).ToDictionary(f => f, f => f.GetTypeTitle());

                this.UpdateBinding(TypesBindingName, types);
                this.UpdateBinding(SelectedTypeBindingName, types.Keys.First());
            }
        }



        private void noTypesToAddCodeActivity_ShowMessage_ExecuteCode(object sender, EventArgs e)
        {
            this.ShowMessage(
                DialogType.Message,
                "${Composite.Management, AssociatedDataElementProviderHelper.AddMetaDataWorkflow.NoTypesTitle}",
                "${Composite.Management, AssociatedDataElementProviderHelper.AddMetaDataWorkflow.NoTypesMessage}"
                );
        }



        private void createFieldGroupCodeActivity_Initialize_ExecuteCode(object sender, EventArgs e)
        {
            Type type = this.GetBinding<Type>(SelectedTypeBindingName);

            string metaDataDefinitionName = type.GetTypeTitle();
            int counter = 1;
            while (PageMetaDataFacade.IsDefinitionAllowed(GetCurrentPageId(), metaDataDefinitionName, metaDataDefinitionName, type.GetImmutableTypeId()) == false)
            {
                metaDataDefinitionName = string.Format("{0} {1}", type.GetTypeTitle(), counter++);
            }

            this.UpdateBinding(FieldGroupNameBindingName, metaDataDefinitionName);
            this.UpdateBinding(FieldGroupLabelBindingName, metaDataDefinitionName);


            if (this.BindingExist(ContainersBindingName) == false)
            {
                List<KeyValuePair<Guid, string>> containers = PageMetaDataFacade.GetAllMetaDataContainers();
                this.UpdateBinding(ContainersBindingName, containers);
                this.UpdateBinding(SelectedContainerBindingName, containers.First().Key);

                Dictionary<int, string> startDisplayOptions = new Dictionary<int, string>();
                if (GetCurrentPage() != null)
                {
                    startDisplayOptions.Add(0, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.StartDisplayOption0"));
                }
                startDisplayOptions.Add(1, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.StartDisplayOption1"));
                startDisplayOptions.Add(2, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.StartDisplayOption2"));
                startDisplayOptions.Add(3, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.StartDisplayOption3"));
                startDisplayOptions.Add(4, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.StartDisplayOption4"));
                startDisplayOptions.Add(5, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.StartDisplayOption5"));
                this.UpdateBinding(StartDisplayOptionsBindingName, startDisplayOptions);
                this.UpdateBinding(SelectedStartDisplayBindingName, startDisplayOptions.Keys.First());

                Dictionary<int, string> inheritDisplayOptions = new Dictionary<int, string>();
                inheritDisplayOptions.Add(0, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.InheritDisplayOption0"));
                inheritDisplayOptions.Add(1, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.InheritDisplayOption1"));
                inheritDisplayOptions.Add(2, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.InheritDisplayOption2"));
                inheritDisplayOptions.Add(3, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.InheritDisplayOption3"));
                inheritDisplayOptions.Add(10000, StringResourceSystemFacade.GetString("Composite.Management", "AssociatedDataElementProviderHelper.AddMetaDataWorkflow.InheritDisplayOption4"));
                this.UpdateBinding(InheritDisplayOptionsBindingName, inheritDisplayOptions);
                this.UpdateBinding(SelectedInheritDisplayBindingName, inheritDisplayOptions.Keys.Last());
            }
        }



        private void createFieldGroupCodeActivity_CreateVisabilatiyRule_ExecuteCode(object sender, EventArgs e)
        {
            string metaDataDefinitionName = this.GetBinding<string>(FieldGroupNameBindingName);
            string metaDataDefinitionLabel = this.GetBinding<string>(FieldGroupLabelBindingName);
            Type type = this.GetBinding<Type>(SelectedTypeBindingName);

            if (PageMetaDataFacade.IsDefinitionAllowed(GetCurrentPageId(), metaDataDefinitionName, metaDataDefinitionLabel, type.GetImmutableTypeId()))
            {
                int startLevel = this.GetBinding<int>(SelectedStartDisplayBindingName);
                int levels = this.GetBinding<int>(SelectedInheritDisplayBindingName);
                PageMetaDataDescription dataAssociationVisabilityRule = PageMetaDataDescription.Branch(startLevel, levels);

                this.UpdateBinding(DataAssociationVisabilityDescriptionBindingName, dataAssociationVisabilityRule);
            }
            else
            {
                this.UpdateBinding(DataAssociationVisabilityDescriptionBindingName, null);

                this.ShowFieldMessage(FieldGroupNameBindingName, "${Composite.Management, AssociatedDataElementProviderHelper.AddMetaDataWorkflow.FieldGroupNameNotValid}");
            }
        }



        private void enterDefaultValuesCodeActivity_Initialize_ExecuteCode(object sender, EventArgs e)
        {
            Type type = this.GetBinding<Type>(SelectedTypeBindingName);
            DataTypeDescriptor dataTypeDescriptor = DynamicTypeManager.GetDataTypeDescriptor(type.GetImmutableTypeId());

            DataTypeDescriptorFormsHelper helper = new DataTypeDescriptorFormsHelper(dataTypeDescriptor);
            helper.LayoutIconHandle = "associated-data-add";

            GeneratedTypesHelper generatedTypesHelper = new GeneratedTypesHelper(dataTypeDescriptor);
            helper.AddReadOnlyFields(generatedTypesHelper.NotEditableDataFieldDescriptorNames);

            IData newData = DataFacade.BuildNew(type);

            helper.UpdateWithNewBindings(this.Bindings);
            helper.ObjectToBindings(newData, this.Bindings);

            this.DeliverFormData(
                    type.Name,
                    StandardUiContainerTypes.Wizard,
                    helper.GetForm(),
                    this.Bindings,
                    helper.GetBindingsValidationRules(newData)
                );
        }



        private void finalizeCodeActivity_Finalize_ExecuteCode(object sender, EventArgs e)
        {
            Type selectedMetaDataType = this.GetBinding<Type>(SelectedTypeBindingName);
            DataTypeDescriptor dataTypeDescriptor = DynamicTypeManager.GetDataTypeDescriptor(selectedMetaDataType.GetImmutableTypeId());

            PageMetaDataDescription dataAssociationVisabilityRule = this.GetBinding<PageMetaDataDescription>(DataAssociationVisabilityDescriptionBindingName);
            Guid metaDataContainerId = this.GetBinding<Guid>(SelectedContainerBindingName);
            string metaDataDefinitionName = this.GetBinding<string>(FieldGroupNameBindingName);
            string metaDataDefinitionLabel = this.GetBinding<string>(FieldGroupLabelBindingName);

            IData newDataTemplate = null;
            if (IsAnyPagesAffected())
            {
                DataTypeDescriptorFormsHelper helper = new DataTypeDescriptorFormsHelper(dataTypeDescriptor);

                GeneratedTypesHelper generatedTypesHelper = new GeneratedTypesHelper(dataTypeDescriptor);
                helper.AddReadOnlyFields(generatedTypesHelper.NotEditableDataFieldDescriptorNames);

                newDataTemplate = DataFacade.BuildNew(selectedMetaDataType);
                helper.BindingsToObject(this.Bindings, newDataTemplate);
            }
            

            using (TransactionScope transactionScope = TransactionsFacade.CreateNewScope())
            {
                IPage page = GetCurrentPage();

                page.AddMetaDataDefinition(metaDataDefinitionName, metaDataDefinitionLabel, selectedMetaDataType.GetImmutableTypeId(), metaDataContainerId, dataAssociationVisabilityRule.StartLevel, dataAssociationVisabilityRule.Levels);

                if (newDataTemplate != null)
                {
                    page.AddNewMetaDataToExistingPages(metaDataDefinitionName, newDataTemplate);
                }

                transactionScope.Complete();
            }

            ParentTreeRefresher parentTreeRefresher = this.CreateParentTreeRefresher();
            parentTreeRefresher.PostRefreshMesseges(this.EntityToken);
        }



        private void MetaDataValid(object sender, ConditionalEventArgs e)
        {
            bool valid = ValidateBindings();

            Type selectedMetaDataType = this.GetBinding<Type>(SelectedTypeBindingName);
            DataTypeDescriptor dataTypeDescriptor = DynamicTypeManager.GetDataTypeDescriptor(selectedMetaDataType.GetImmutableTypeId());

            DataTypeDescriptorFormsHelper helper = new DataTypeDescriptorFormsHelper(dataTypeDescriptor);

            GeneratedTypesHelper generatedTypesHelper = new GeneratedTypesHelper(dataTypeDescriptor);
            helper.AddReadOnlyFields(generatedTypesHelper.NotEditableDataFieldDescriptorNames);

            IData newDataTemplate = DataFacade.BuildNew(selectedMetaDataType);

            if(!BindAndValidate(helper, newDataTemplate))
            {
                valid = false;
            }

            if (valid)
            {
                var fieldsWithInvalidForeginKey = new List<string>();
                DataReferenceFacade.TryValidateForeignKeyIntegrity(newDataTemplate, fieldsWithInvalidForeginKey);
                if (fieldsWithInvalidForeginKey.Count > 0)
                {
                    foreach (string fieldName in fieldsWithInvalidForeginKey)
                    {
                        if (!generatedTypesHelper.NotEditableDataFieldDescriptorNames.Contains(fieldName))
                        {
                            this.ShowFieldMessage(fieldName, "Invalid reference");
                            valid = false;
                        }
                    }

                }
            }

            e.Result = valid;
        }
    }
}
