/*

The contents of this web application are subject to the Mozilla Public License Version 1.1 (the "License"); you may not use this web application except in compliance with the License. 
You may obtain a copy of the License at http://www.mozilla.org/MPL/.

Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
See the License for the specific language governing rights and limitations under the License.

The Original Code is owned by and the Initial Developer of the Original Code is Composite A/S (Danish business reg.no. 21744409). All Rights Reserved

Section 11 of the License is EXPRESSLY amended to include a provision stating that any dispute, including but not limited to disputes related to the enforcement of the License, to which Composite A/S as owner of the Original Code, as Initial Developer or in any other role, becomes a part to shall be governed by Danish law and be initiated before the Copenhagen City Court ("K�benhavns Byret")
                        
*/

using System;
using System.Collections.Generic;
using Composite.C1Console.Security;
using Composite.C1Console.Events;
using Composite.Core.Logging;


namespace Composite.C1Console.Actions.Foundation
{
    internal static class FlowControllerCache
    {
        private static Dictionary<Type, IFlowController> _flowControllerCache = new Dictionary<Type, IFlowController>();

        private static object _lock = new object();



        static FlowControllerCache()
        {
            GlobalEventSystemFacade.SubscribeToFlushEvent(OnFlush);
        }



        public static IFlowController GetFlowController(FlowToken flowToken, FlowControllerServicesContainer flowControllerServicesContainer)
        {
            if (flowToken == null) throw new ArgumentNullException("flowToken");


            IFlowController flowController;


            if (_flowControllerCache.TryGetValue(flowToken.GetType(), out flowController) == false)
            {
                object[] attributes = flowToken.GetType().GetCustomAttributes(typeof(FlowControllerAttribute), true);

                if (attributes.Length == 0) throw new InvalidOperationException(string.Format("Missing '{0}' attribute on the flow token '{1}'", typeof(FlowControllerAttribute), flowToken.GetType()));

                FlowControllerAttribute attribute = (FlowControllerAttribute)attributes[0];

                if (attribute.FlowControllerType == null) throw new InvalidOperationException(string.Format("Flow controller type can not be null on the action token '{0}'", flowToken.GetType()));
                if (typeof(IFlowController).IsAssignableFrom(attribute.FlowControllerType) == false) throw new InvalidOperationException(string.Format("Flow controller '{0}' should implement the interface '{1}'", attribute.FlowControllerType, typeof(IFlowController)));

                flowController = (IFlowController)Activator.CreateInstance(attribute.FlowControllerType);
                flowController.ServicesContainer = flowControllerServicesContainer;

                _flowControllerCache.Add(flowToken.GetType(), flowController);
            }


            return flowController;
        }



        private static void Flush()
        {
            _flowControllerCache = new Dictionary<Type, IFlowController>();
        }



        private static void OnFlush(FlushEventArgs args)
        {
            Flush();
        }
    }
}
