/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using Composite.C1Console.Actions;
using Composite.C1Console.Security;


namespace Composite.C1Console.Elements.Plugins.ElementProvider
{
    /// <summary>
    /// You can implement this interface on your element provider to handle the "drag and drop" event.
    /// </summary>
    public interface IDragAndDropElementProvider : IHooklessElementProvider
    {
        /// <summary>
        /// Invoked when an element is dropped
        /// </summary>
        /// <param name="draggedEntityToken">The <see cref="EntityToken"/> of the element being dragged</param>
        /// <param name="newParentEntityToken">The <see cref="EntityToken"/> of the element receiving the dragged item</param>
        /// <param name="dropIndex">The index (position) the element was dropped</param>
        /// <param name="dragAndDropType">The type identifying the drang and drop action</param>
        /// <param name="draggedElementFlowControllerServicesContainer">handle that let you communicate to client via services (like popping a dialog)</param>
        /// <returns>True is action completed as expected and tree should be updated on the client</returns>
        bool OnElementDraggedAndDropped(EntityToken draggedEntityToken, EntityToken newParentEntityToken, int dropIndex, DragAndDropType dragAndDropType, FlowControllerServicesContainer draggedElementFlowControllerServicesContainer);
    }


    /// <summary>
    /// Specifies the kind of action that should happen on drop
    /// </summary>
    public enum DragAndDropType
    {
        /// <summary>
        /// The element is moved
        /// </summary>
        Move,
        /// <summary>
        /// The element is copied
        /// </summary>
        Copy
    }
}
