/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System.IO;
using Composite.Core.IO;
using Composite.Core.IO.Plugins.IOProvider;


namespace Composite.Core.Implementation
{
    /// <summary>
    /// Implementation of <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
    /// </summary>
    public class C1FileSystemWatcherImplementation
    {
        private IC1FileSystemWatcher _fileSystemWatcher;


        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        /// <param name="path"></param>
        /// <param name="filter"></param>
        public C1FileSystemWatcherImplementation(string path, string filter)
        {
            _fileSystemWatcher = IOFacade.CreateC1FileSystemWatcher(path, filter);
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual bool EnableRaisingEvents
        {
            get
            {
                return _fileSystemWatcher.EnableRaisingEvents;
            }
            set
            {
                _fileSystemWatcher.EnableRaisingEvents = value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual string Path
        {
            get
            {
                return _fileSystemWatcher.Path;
            }
            set
            {
                _fileSystemWatcher.Path = value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual string Filter
        {
            get
            {
                return _fileSystemWatcher.Filter;
            }
            set
            {
                _fileSystemWatcher.Filter = value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual bool IncludeSubdirectories
        {
            get
            {
                return _fileSystemWatcher.IncludeSubdirectories;
            }
            set
            {
                _fileSystemWatcher.IncludeSubdirectories = value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual event FileSystemEventHandler Created
        {
            add
            {
                _fileSystemWatcher.Created += value;
            }
            remove
            {
                _fileSystemWatcher.Created -= value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual event FileSystemEventHandler Changed
        {
            add
            {
                _fileSystemWatcher.Changed += value;
            }
            remove
            {
                _fileSystemWatcher.Changed -= value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual event RenamedEventHandler Renamed
        {
            add
            {
                _fileSystemWatcher.Renamed += value;
            }
            remove
            {
                _fileSystemWatcher.Renamed -= value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual event FileSystemEventHandler Deleted
        {
            add
            {
                _fileSystemWatcher.Deleted += value;
            }
            remove
            {
                _fileSystemWatcher.Deleted -= value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1716:IdentifiersShouldNotMatchKeywords", MessageId = "Error")]
        public virtual event ErrorEventHandler Error
        {
            add
            {
                _fileSystemWatcher.Error += value;
            }
            remove
            {
                _fileSystemWatcher.Error -= value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual NotifyFilters NotifyFilter
        {
            get
            {
                return _fileSystemWatcher.NotifyFilter;
            }
            set
            {
                _fileSystemWatcher.NotifyFilter = value;
            }
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual void BeginInit()
        {
            _fileSystemWatcher.BeginInit();
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        public virtual void EndInit()
        {
            _fileSystemWatcher.EndInit();
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        /// <param name="changeType"></param>
        /// <returns></returns>
        public virtual C1WaitForChangedResult WaitForChanged(WatcherChangeTypes changeType)
        {
            return _fileSystemWatcher.WaitForChanged(changeType);
        }



        /// <summary>
        /// See <see cref="Composite.Core.IO.C1FileSystemWatcher"/>.
        /// </summary>
        /// <param name="changeType"></param>
        /// <param name="timeout"></param>
        /// <returns></returns>
        public virtual C1WaitForChangedResult WaitForChanged(WatcherChangeTypes changeType, int timeout)
        {
            return _fileSystemWatcher.WaitForChanged(changeType, timeout);
        }
    }
}
