/*

The contents of this web application are subject to the Mozilla Public License Version 1.1 (the "License"); you may not use this web application except in compliance with the License. 
You may obtain a copy of the License at http://www.mozilla.org/MPL/.

Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
See the License for the specific language governing rights and limitations under the License.

The Original Code is owned by and the Initial Developer of the Original Code is Composite A/S (Danish business reg.no. 21744409). All Rights Reserved

Section 11 of the License is EXPRESSLY amended to include a provision stating that any dispute, including but not limited to disputes related to the enforcement of the License, to which Composite A/S as owner of the Original Code, as Initial Developer or in any other role, becomes a part to shall be governed by Danish law and be initiated before the Copenhagen City Court ("K�benhavns Byret")
                        
*/

using System;
using System.Diagnostics;
using System.Reflection;


namespace Composite.Core.Instrumentation.Foundation
{
    internal sealed class XmlTimerProfiler : TimerProfiler
	{
        private int _startTime;
        private bool _disposed = false;


        internal XmlTimerProfiler()
        {
            StackTrace stackTrace = new StackTrace();

            StackFrame stackFrame = stackTrace.GetFrame(2);

            MemberInfo methodInfo = stackFrame.GetMethod();

            string methodName = string.Format("{0}.{1}", methodInfo.DeclaringType.FullName, methodInfo.Name);

            _startTime = Environment.TickCount;

            XmlTimerProfilerFacade.Instance.Push(methodName, null);
        }



        internal XmlTimerProfiler(string message)
        {
            StackTrace stackTrace = new StackTrace();

            StackFrame stackFrame = stackTrace.GetFrame(2);

            MemberInfo methodInfo = stackFrame.GetMethod();

            string methodName = string.Format("{0}.{1}", methodInfo.DeclaringType.FullName, methodInfo.Name);

            _startTime = Environment.TickCount;

            XmlTimerProfilerFacade.Instance.Push(methodName, message);
        }



        public override void Dispose()
        {
            if (_disposed == false)
            {
                int elapsedTime = Environment.TickCount - _startTime;

                XmlTimerProfilerFacade.Instance.Pop(elapsedTime);

                _disposed = true;
            }
        }



        ~XmlTimerProfiler()
        {
            Dispose();
        }
	}
}
