/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Xml.Linq;
using Composite.Core.Xml;


namespace Composite.Core.Instrumentation.Foundation
{
    internal sealed class XmlTimerProfilerFacade : IDisposable
    {
        private static XmlTimerProfilerFacade _instance = null;
        private static readonly object _lock = new object();


        public Stack<XElement> _elementStack = new Stack<XElement>();


        private XmlTimerProfilerFacade()
        {
            Composite.Core.Logging.LoggingService.LogVerbose("XmlTimerProfilerFacade", "XmlTimerProfiler running");
            _elementStack.Push(new XElement("Root"));
        }



        internal static XmlTimerProfilerFacade Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (_lock)
                    {
                        if (_instance == null)
                        {
                            _instance = new XmlTimerProfilerFacade();
                        }
                    }
                }

                return _instance;
            }
        }



        internal void Push(string methodName, string message)
        {
            lock (_lock)
            {
                XElement element = _elementStack.Peek();
                
                XElement newElement = new XElement(methodName.Replace('`', '_').Replace('+', '_').Replace('<', '_').Replace('>', '_'));
                if (message != null)
                {
                    newElement.Add(new XAttribute("message", message));
                }
                _elementStack.Push(newElement);

                //System.Threading.Thread.CurrentThread.ManagedThreadId
                element.Add(newElement);
            }
        }



        internal void Pop(int elapsedTime)
        {
            lock (_lock)
            {
                XElement element = _elementStack.Pop();

                element.Add(new XAttribute("elapsedTime", elapsedTime));
            }
        }        



        public void Dispose()
        {
            try
            {
                XElement element = _elementStack.Pop();

                XDocument doc = new XDocument(element);

                // avoid writes to windows/system32 dir on web
                doc.SaveToFile(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "TimerProfile.xml"));
            }
            catch( Exception ex)
            {
                Composite.Core.Logging.LoggingService.LogCritical("XmlTimerProfilerFacade", ex);
            }
        }



        ~XmlTimerProfilerFacade()
        {
            Dispose();
        }
    }
}
