/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Web;
using Composite.Core.Extensions;


namespace Composite.Core.WebClient
{
    /// <summary>    
    /// </summary>
    /// <exclude />
    [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)] 
    [Obsolete("Use Composite.UrlBuilder")]
    public sealed class UrlString
    {
        private static readonly string IncorrectValueParam = "__***IncorrectValue***__";

        private string _pathInfo;
        private string _filePath;
        private List<KeyValuePair<string, string>> _queryParameters;


        /// <exclude />
        public UrlString(string url)
        {
            _queryParameters = new List<KeyValuePair<string, string>>();

            int questionMarkIndex = url.IndexOf("?");
            if (questionMarkIndex < 0)
            {
                ExtractPathInfo(url, out _filePath, out _pathInfo);
                return;
            }

            ExtractPathInfo(url.Substring(0, questionMarkIndex), out _filePath, out _pathInfo);

            if (questionMarkIndex + 1 == url.Length)
            {
                return;
            }

            string queryParamStr = url.Substring(questionMarkIndex + 1, url.Length - questionMarkIndex - 1);
            foreach (string queryParam in queryParamStr.Split(new[] { "&amp;", "&" }, StringSplitOptions.RemoveEmptyEntries))
            {
                string[] parts = queryParam.Split(new[] { '=' });

                bool badUrl = parts.Length != 2;
                if (!badUrl)
                {
                    string encodedKey = parts[0];
                    string encodedValue = parts[1];

                    string key = HttpUtility.UrlDecode(encodedKey);
                    string value = HttpUtility.UrlDecode(encodedValue);

                    badUrl = HttpUtility.UrlEncode(key) != encodedKey
                             || HttpUtility.UrlEncode(value) != encodedValue;

                    if (!badUrl)
                    {
                        _queryParameters.Add(new KeyValuePair<string, string>(key, value));
                        continue;
                    }
                }

                _queryParameters.Add(new KeyValuePair<string, string>(queryParam, IncorrectValueParam));
            }
        }



        private void ExtractPathInfo(string relativePath, out string filePath, out string pathInfo)
        {
            int aspxExtOffset = relativePath.IndexOf(".aspx");
            if (aspxExtOffset < 0 || aspxExtOffset == relativePath.Length - 5)
            {
                pathInfo = null;
                filePath = relativePath;
                return;
            }
            filePath = relativePath.Substring(0, aspxExtOffset + 5);
            pathInfo = relativePath.Substring(aspxExtOffset + 5);
        }



        /// <exclude />
        public override string ToString()
        {
            // NOTE: StringBuilder shouldn't be used - it is to slow
            string queryString = QueryString;

            string result = _filePath;
            if (_pathInfo != null)
            {
                result += _pathInfo;
            }

            if (queryString != string.Empty)
            {
                result += "?" + queryString;
            }

            return result; // _filePath + _pathInfo + "?" + queryString
        }



        /// <exclude />
        public void AddQueryParameters(NameValueCollection parameters)
        {
            foreach (string key in parameters)
            {
                this[key] = parameters[key];
            }
        }



        /// <exclude />
        public NameValueCollection GetQueryParameters()
        {
            var result = new NameValueCollection();
            foreach (KeyValuePair<string, string> pair in _queryParameters)
            {
                result.Add(pair.Key, pair.Value);
            }
            return result;
        }



        /// <exclude />
        public string this[string key]
        {
            get
            {
                string value = _queryParameters.Where(pair => pair.Key == key).Select(pair => pair.Value).FirstOrDefault();
                return value ?? string.Empty;
            }
            set
            {
                for (int i = 0; i < _queryParameters.Count; i++)
                {
                    if (_queryParameters[i].Key == key)
                    {
                        if (value == null)
                        {
                            _queryParameters.RemoveAt(i);
                        }
                        else
                        {
                            _queryParameters[i] = new KeyValuePair<string, string>(key, value);
                        }

                        return;
                    }
                }

                if (value != null)
                {
                    _queryParameters.Add(new KeyValuePair<string, string>(key, value));
                }
            }
        }



        /// <exclude />
        public string PathInfo
        {
            get
            {
                return _pathInfo ?? string.Empty;
            }
            set
            {
                _pathInfo = value;
            }
        }



        /// <exclude />
        public string FilePath
        {
            get
            {
                return _filePath;
            }
            set
            {
                Verify.ArgumentNotNull(value, "value");
                _filePath = value;
            }
        }



        /// <exclude />
        public string ServerUrl
        {
            get
            {
                if (_filePath.IsNullOrEmpty())
                {
                    return string.Empty;
                }

                int index1 = _filePath.IndexOf("://");
                if (index1 <= 0 || _filePath.Length == index1 + 4)
                {
                    return string.Empty;
                }

                int index2 = _filePath.IndexOf("/", index1 + 3);
                if (index2 < 0)
                {
                    return string.Empty;
                }

                return _filePath.Substring(0, index2 + 1);
            }
            set
            {
                if (!ServerUrl.IsNullOrEmpty())
                {
                    throw new NotImplementedException();
                }

                if (value.IsNullOrEmpty()) return;

                Verify.IsTrue(value.EndsWith("/"), "Wrong server url string");

                if (_filePath.StartsWith("/")) _filePath = _filePath.Substring(1);

                _filePath = value + _filePath;
            }
        }



        /// <exclude />
        public string QueryString
        {
            get
            {
                if (_queryParameters.Count == 0)
                {
                    return string.Empty;
                }

                var sb = new StringBuilder();
                for (int i = 0; i < _queryParameters.Count; i++)
                {
                    if (i != 0)
                    {
                        sb.Append("&");
                    }

                    if (_queryParameters[i].Value == IncorrectValueParam)
                    {
                        sb.Append(_queryParameters[i].Key);
                    }
                    else
                    {
                        sb.Append(HttpUtility.UrlEncode(_queryParameters[i].Key));
                        sb.Append('=');
                        sb.Append(HttpUtility.UrlEncode(_queryParameters[i].Value));
                    }
                }

                return sb.ToString();
            }
        }
    }
}
