/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Composite.C1Console.Events;
using Composite.Core.Logging;


namespace Composite.Data.Caching.Foundation
{
    internal static class CachingQueryableCache
    {
        private static Dictionary<Type, MethodInfo> _cachingQueryableGetEnumeratorMethodInfoCache = new Dictionary<Type, MethodInfo>();
        private static Dictionary<Type, Type> _cachingQueryableTypeCache = new Dictionary<Type, Type>();
        private static Dictionary<Type, Dictionary<Type, MethodInfo>> _cachingQueryableExecuteMethodInfoCache = new Dictionary<Type, Dictionary<Type, MethodInfo>>();


        static CachingQueryableCache()
        {
            GlobalEventSystemFacade.SubscribeToFlushEvent(OnFlush);
        }



        public static MethodInfo GetCachingQueryableGetEnumeratorMethodInfo(Type genericType)
        {
            MethodInfo methodInfo;

            if (_cachingQueryableGetEnumeratorMethodInfoCache.TryGetValue(genericType, out methodInfo) == false)
            {
                Type type = typeof(CachingQueryable<>).MakeGenericType(new Type[] { genericType });

                methodInfo =
                    (from method in type.GetMethods()
                     where method.Name == "GetEnumerator"
                     select method).First();

                _cachingQueryableGetEnumeratorMethodInfoCache.Add(genericType, methodInfo);
            }

            return methodInfo;
        }



        public static Type GetCachingQueryableType(Type elementType)
        {
            Type type;

            if (_cachingQueryableTypeCache.TryGetValue(elementType, out type) == false)
            {
                type = typeof(CachingQueryable<>).MakeGenericType(new Type[] { elementType });

                _cachingQueryableTypeCache.Add(elementType, type);
            }

            return type;
        }



        public static MethodInfo GetCachingQueryableExecuteMethodInfo(Type genericType, Type expressionType)
        {
            Dictionary<Type, MethodInfo> methodInfoes;

            if (_cachingQueryableExecuteMethodInfoCache.TryGetValue(genericType, out methodInfoes) == false)
            {
                methodInfoes = new Dictionary<Type, MethodInfo>();

                _cachingQueryableExecuteMethodInfoCache.Add(genericType, methodInfoes);
            }


            MethodInfo methodInfo;

            if (methodInfoes.TryGetValue(expressionType, out methodInfo) == false)
            {
                Type type = typeof(CachingQueryable<>).MakeGenericType(new Type[] { genericType });

                methodInfo =
                    (from method in type.GetMethods()
                     where method.Name == "Execute" &&
                           method.IsGenericMethod 
                     select method).First();

                methodInfo = methodInfo.MakeGenericMethod(new Type[] { expressionType });

                methodInfoes.Add(expressionType, methodInfo);
            }

            return methodInfo;
        }



        private static void Flush()
        {
            _cachingQueryableGetEnumeratorMethodInfoCache = new Dictionary<Type, MethodInfo>();
            _cachingQueryableTypeCache = new Dictionary<Type, Type>();
            _cachingQueryableExecuteMethodInfoCache = new Dictionary<Type, Dictionary<Type, MethodInfo>>();
        }



        private static void OnFlush(FlushEventArgs args)
        {
            Flush();
        }
    }
}
