/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.Globalization;
using Composite.Data.ProcessControlled;


namespace Composite.Data
{
	internal interface IDataLocalizationFacade
	{
        /// <summary>
        /// This method return true if the system should use localization functionallity.
        /// In ohter words, if there are two or more languages added then localization functionallity
        /// should be used.
        /// </summary>
        bool UseLocalization { get; }

        IEnumerable<CultureInfo> WhiteListedLocales { get; }

        CultureInfo DefaultLocalizationCulture { get; set; }
        CultureInfo DefaultUrlMappingCulture { get; }
        IEnumerable<string> ActiveLocalizationNames { get; }

        string GetUrlMappingName(CultureInfo cultureInfo);
        CultureInfo GetCultureInfoByUrlMappingName(string urlMappingName);
        IEnumerable<string> UrlMappingNames { get; }

        bool IsLocalized(Type type);
        bool IsLocalizable(Type type);

        IEnumerable<ReferenceFailingPropertyInfo> GetReferencingLocalizeFailingProperties(ILocalizedControlled data);

	    string GetCultureTitle(CultureInfo culture);
	}
}
