/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;


namespace Composite.Data.Types
{
    
    /// <summary>    
    /// Page meta data definition. 
    /// </summary>
    /// <remarks>
    /// Using the same name for a metadata definition is allowed if metadata type and label are the same
    /// on all instances.
    /// </remarks>
    /// <exclude />
    [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)] 
    [AutoUpdateble]
    [KeyPropertyName("Id")]
    [ImmutableTypeId("{F0101D4E-2EC2-4D24-B0BD-BE367DC7C3E1}")]
    [CachingAttribute(CachingType.Full)]
    [DataScope(DataScopeIdentifier.PublicName)]
    public interface IPageMetaDataDefinition : IData
    {
        /// <exclude />
        [StoreFieldType(PhysicalStoreFieldType.Guid)]
        [DefaultFieldNewGuidValue()]
        [ImmutableFieldId("{C30E50F4-64BF-46A8-B8C9-A8F8CA62C74D}")]
        Guid Id { get; set; }


        /// <summary>
        /// This is an id of a page or pagetype item and a reference to the
        /// elemen where this description is defined.
        /// This should be an id of a page or pagetype og Guid.Empty for 
        /// the whole website
        /// </summary>
        [StoreFieldType(PhysicalStoreFieldType.Guid)]
        [ImmutableFieldId("{5C8A6831-8B94-424c-90C7-7C9385E5DB7C}")]
        Guid DefiningItemId { get; set; }


        /// <exclude />
        [StoreFieldType(PhysicalStoreFieldType.String, 128)]
        [ImmutableFieldId("{8B421E9F-F0B5-4D27-B1E9-87D6814EAC0F}")]
        string Name { get; set; }


        /// <exclude />
        [StoreFieldType(PhysicalStoreFieldType.String, 256)]
        [ImmutableFieldId("{3838006A-88D1-485F-8ED6-46E14E6A738B}")]
        string Label { get; set; }


        /// <exclude />
        [StoreFieldType(PhysicalStoreFieldType.Guid)]
        [ImmutableFieldId("{9EA6FEA8-B2E7-4F44-A5F5-0169123CEC77}")]
        [ForeignKey(typeof(ICompositionContainer), "Id", AllowCascadeDeletes = true)]
        Guid MetaDataContainerId { get; set; }


        /// <exclude />
        [StoreFieldType(PhysicalStoreFieldType.Guid)]
        [ImmutableFieldId("{1AD2B5F6-F5AE-496A-B9FA-47BF5E90F5F2}")]
        Guid MetaDataTypeId { get; set; }


        /// <summary>
        /// Start level number relativ to the page where this is defined.
        /// 0 will include the page it self
        /// </summary>
        [StoreFieldType(PhysicalStoreFieldType.Integer)]
        [ImmutableFieldId("{8AC8520E-97B9-4F58-B9A4-D5ED33C482EC}")]
        int StartLevel { get; set; }


        /// <summary>
        /// The number of levels this definition affects
        /// Use int.Max for branch
        /// </summary>
        [StoreFieldType(PhysicalStoreFieldType.Integer)]
        [ImmutableFieldId("{C76C80E0-C8DF-4A7B-8931-D3C3B229F44E}")]
        int Levels { get; set; }
    }




    /// <summary>    
    /// </summary>
    /// <exclude />
    [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)] 
    public sealed class PageMetaDataDefinitionEqualityComparer : IEqualityComparer<IPageMetaDataDefinition>
    {
        /// <exclude />
        public bool Equals(IPageMetaDataDefinition x, IPageMetaDataDefinition y)
        {
            return x.Name == y.Name && x.MetaDataTypeId == y.MetaDataTypeId;
        }


        /// <exclude />
        public int GetHashCode(IPageMetaDataDefinition obj)
        {
            return obj.Name.GetHashCode() ^ obj.MetaDataTypeId.GetHashCode();
        }
    }
}
