/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.Xml.Linq;
using Composite.Functions.Foundation;
using Composite.Core.Types;


namespace Composite.Functions
{
    internal sealed class ConstantParameterRuntimeTreeNode : BaseParameterRuntimeTreeNode
    {
        private object _constantValue;
        private bool _isEnumerable = false;


        public ConstantParameterRuntimeTreeNode(string name, string constantValue)
            : base(name)
        {
            _constantValue = constantValue;
        }


        public ConstantParameterRuntimeTreeNode(string name, IEnumerable<string> constantValue)
            : base(name)
        {
            _constantValue = constantValue;
            _isEnumerable = true;
        }



        public override object GetValue(FunctionContextContainer contextContainer)
        {
            if (contextContainer == null) throw new ArgumentNullException("contextContainer");

            return _constantValue;
        }


        public override IEnumerable<string> GetAllSubFunctionNames()
        {
            yield break;
        }


        public override bool ContainsNestedFunctions
        {
            get
            {
                return false;
            }
        }



        public override XElement Serialize()
        {
            if (_isEnumerable)
            {
                XElement element = 
                    new XElement(XName.Get(FunctionTreeConfigurationNames.ParamTagName, FunctionTreeConfigurationNames.NamespaceName),
                        new XAttribute(FunctionTreeConfigurationNames.NameAttributeName, this.Name)
                    );

                IEnumerable<string> strings = (IEnumerable<string>)_constantValue;
                foreach (string s in strings)
                {
                    element.Add(new XElement(
                            XName.Get(FunctionTreeConfigurationNames.ParamElementTagName, FunctionTreeConfigurationNames.NamespaceName),
                            new XAttribute(FunctionTreeConfigurationNames.ValueAttributeName, s)                            
                        ));
                }

                return element;
            }
            else
            {
                if (_constantValue != null)
                {
                    XElement element =
                        new XElement(XName.Get(FunctionTreeConfigurationNames.ParamTagName, FunctionTreeConfigurationNames.NamespaceName),
                            new XAttribute(FunctionTreeConfigurationNames.NameAttributeName, this.Name),
                            new XAttribute(FunctionTreeConfigurationNames.ValueAttributeName, _constantValue)
                        );

                    return element;
                }
                else
                {
                    XElement element =
                        new XElement(XName.Get(FunctionTreeConfigurationNames.ParamTagName, FunctionTreeConfigurationNames.NamespaceName),
                            new XAttribute(FunctionTreeConfigurationNames.NameAttributeName, this.Name)
                        );

                    return element;
                }

            }
        }
    }
}
