/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Reflection;
using System.Text;
using Composite.C1Console.Actions;
using Composite.C1Console.Elements;
using Composite.C1Console.Elements.ElementProviderHelpers.DataGroupingProviderHelper;
using Composite.C1Console.Elements.Plugins.ElementProvider;
using Composite.C1Console.Events;
using Composite.C1Console.Security;
using Composite.C1Console.Users;
using Composite.C1Console.Workflow;
using Composite.Core;
using Composite.Core.Extensions;
using Composite.Core.Linq;
using Composite.Core.ResourceSystem;
using Composite.Core.ResourceSystem.Icons;
using Composite.Core.Serialization;
using Composite.Core.Types;
using Composite.Core.WebClient;
using Composite.Data;
using Composite.Data.DynamicTypes;
using Composite.Data.ProcessControlled;
using Composite.Data.ProcessControlled.ProcessControllers.GenericPublishProcessController;
using Composite.Data.Types;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;
using Microsoft.Practices.ObjectBuilder;

using Texts = Composite.Core.ResourceSystem.LocalizationFiles.Composite_Plugins_GeneratedDataTypesElementProvider;

namespace Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider
{
    #region ToXml
    internal sealed class DataTypeDescriptorToXmlActionExecutor : IActionExecutor
    {
        public FlowToken Execute(EntityToken entityToken, ActionToken actionToken, FlowControllerServicesContainer flowControllerServicesContainer)
        {
            GeneratedDataTypesElementProviderTypeEntityToken castedEntityToken = (GeneratedDataTypesElementProviderTypeEntityToken)entityToken;

            string currentConsoleId = flowControllerServicesContainer.GetService<IManagementConsoleMessageService>().CurrentConsoleId;

            string url = string.Format("{0}?TypeName={1}", UrlUtils.ResolveAdminUrl("content/views/datatypedescriptor/ToXml.aspx"), System.Web.HttpUtility.UrlEncode(castedEntityToken.SerializedTypeName));

            ConsoleMessageQueueFacade.Enqueue(
                new OpenViewMessageQueueItem
                {
                    EntityToken = EntityTokenSerializer.Serialize(entityToken, true),
                    Url = url,
                    ViewId = Guid.NewGuid().ToString(),
                    ViewType = ViewType.Main,
                    Label = GeneratedDataTypesElementProvider.GetText("DataTypeDescriptorToXmlLabel")
                },
                currentConsoleId
            );

            return null;
        }
    }



    /// <summary>    
    /// </summary>
    /// <exclude />
    [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]
    [ActionExecutor(typeof(DataTypeDescriptorToXmlActionExecutor))]
    public sealed class DataTypeDescriptorToXmlActionToken : ActionToken
    {
        private static PermissionType[] _permissionTypes = new PermissionType[] { PermissionType.Administrate };

        /// <exclude />
        public override IEnumerable<PermissionType> PermissionTypes
        {
            get { return _permissionTypes; }
        }

        /// <exclude />
        public override string Serialize()
        {
            return "DataTypeDescriptorToXml";
        }

        /// <exclude />
        public static ActionToken Deserialize(string serializedData)
        {
            return new DataTypeDescriptorToXmlActionToken();
        }
    }
    #endregion



    [ConfigurationElementType(typeof(GeneratedDataTypesElementProviderData))]
    internal sealed class GeneratedDataTypesElementProvider : IHooklessElementProvider, ILocaleAwareElementProvider
    {
        private static readonly string LogTitle = typeof (GeneratedDataTypesElementProvider).Name;

        private ElementProviderContext _providerContext;
        private readonly bool _websiteItemsView;
        private DataGroupingProviderHelper _dataGroupingProviderHelper;


        /// <exclude />
        public static ResourceHandle RootOpen { get { return GetIconHandle("generated-root-open"); } }

        /// <exclude />
        public static ResourceHandle RootClosed { get { return GetIconHandle("generated-root-closed"); } }

        /// <exclude />
        public static ResourceHandle InterfaceOpen { get { return GetIconHandle("generated-interface-open"); } }

        /// <exclude />
        public static ResourceHandle InterfaceClosed { get { return GetIconHandle("generated-interface-closed"); } }

        /// <exclude />
        public static ResourceHandle AddDataTypeIcon { get { return GetIconHandle("generated-type-add"); } }

        /// <exclude />
        public static ResourceHandle EditDataTypeIcon { get { return GetIconHandle("generated-type-edit"); } }

        /// <exclude />
        public static ResourceHandle DeleteDataTypeIcon { get { return GetIconHandle("generated-type-delete"); } }

        /// <exclude />
        public static ResourceHandle LocalizeDataTypeIcon { get { return GetIconHandle("generated-type-localize"); } }

        /// <exclude />
        public static ResourceHandle DelocalizeDataTypeIcon { get { return GetIconHandle("generated-type-delocalize"); } }

        /// <exclude />
        public static ResourceHandle AddDataIcon { get { return GetIconHandle("generated-type-data-add"); } }

        /// <exclude />
        public static ResourceHandle EditDataIcon { get { return GetIconHandle("generated-type-data-edit"); } }

        /// <exclude />
        public static ResourceHandle DeleteDataIcon { get { return GetIconHandle("generated-type-data-delete"); } }

        /// <exclude />
        public static ResourceHandle LocalizeDataIcon { get { return GetIconHandle("generated-type-data-localize"); } }

        /// <exclude />
        public static ResourceHandle ListUnpublishedItemsIcon = GetIconHandle("generated-type-list-unpublished-items");

        /// <exclude />
        public static ResourceHandle ShowincontentareaIcon = GetIconHandle("generated-type-showincontentarea");

        /// <exclude />
        public static ResourceHandle EditFormMarkupIcon { get { return GetIconHandle("generated-type-form-markup-edit"); } }

        /// <exclude />
        public static ResourceHandle ToXmlIcon { get { return GetIconHandle("generated-type-to-xml"); } }

        /// <exclude />
        public static readonly Dictionary<string, ResourceHandle> DataIconLookup;

        /// <exclude />
        public static ResourceHandle ErrorIcon { get { return GetIconHandle("error"); } }

        private static readonly ActionGroup PrimaryActionGroup = new ActionGroup(ActionGroupPriority.PrimaryHigh);
        private static readonly ActionGroup ViewActionGroup = new ActionGroup("View", ActionGroupPriority.PrimaryLow);
        private static readonly ActionGroup AppendedActionGroup = new ActionGroup("Develop", ActionGroupPriority.GeneralAppendMedium);

        private static readonly PermissionType[] _addNewInterfaceTypePermissionTypes = new PermissionType[] { PermissionType.Add };
        private static readonly PermissionType[] _editInterfaceTypePermissionTypes = new PermissionType[] { PermissionType.Edit };
        private static readonly PermissionType[] _editFormMarkupPermissionTypes = new PermissionType[] { PermissionType.Edit };
        private static readonly PermissionType[] _deleteInterfaceTypePermissionTypes = new PermissionType[] { PermissionType.Delete };

        private static readonly PermissionType[] _addNewDataPermissionTypes = new PermissionType[] { PermissionType.Add };
        private static readonly PermissionType[] _editDataPermissionTypes = new PermissionType[] { PermissionType.Edit };
        private static readonly PermissionType[] _deleteDataPermissionTypes = new PermissionType[] { PermissionType.Delete };
        private static readonly PermissionType[] _localizeDataPermissionTypes = new PermissionType[] { PermissionType.Add };



        /// <exclude />
        static GeneratedDataTypesElementProvider()
        {
            DataIconLookup = new Dictionary<string, ResourceHandle>();
            DataIconLookup.Add(GenericPublishProcessController.Draft, DataIconFacade.DataDraftIcon);
            DataIconLookup.Add(GenericPublishProcessController.AwaitingApproval, DataIconFacade.DataAwaitingApprovalIcon);
            DataIconLookup.Add(GenericPublishProcessController.AwaitingPublication, DataIconFacade.DataAwaitingPublicationIcon);
            DataIconLookup.Add(GenericPublishProcessController.Published, DataIconFacade.DataPublishedIcon);
        }



        /// <exclude />
        public GeneratedDataTypesElementProvider(bool onlyShowGlobalDatas)
        {
            _websiteItemsView = onlyShowGlobalDatas;
        }



        /// <exclude />
        public ElementProviderContext Context
        {
            set
            {
                _providerContext = value;
                _dataGroupingProviderHelper = new DataGroupingProviderHelper(_providerContext);
                _dataGroupingProviderHelper.FolderOpenIcon = GetIconHandle("generated-interface-open");
                _dataGroupingProviderHelper.FolderClosedIcon = GetIconHandle("generated-interface-closed");
                _dataGroupingProviderHelper.OnCreateLeafElement = GetElementFromData;
                _dataGroupingProviderHelper.OnCreateGhostedLeafElement = GetGhostedElementFromData;
                _dataGroupingProviderHelper.OnCreateDisabledLeafElement = GetDisabledElementFromData;
                _dataGroupingProviderHelper.OnAddActions = AddGroupFolderActions;
                _dataGroupingProviderHelper.OnGetRootParentEntityToken = GetRootParentEntityToken;
                _dataGroupingProviderHelper.OnOwnsType = type =>
                {
                    if (!type.IsGenerated() && !type.IsStaticDataType()) return false;
                    if (PageFolderFacade.GetAllFolderTypes().Contains(type)) return false;
                    if (PageMetaDataFacade.GetAllMetaDataTypes().Contains(type)) return false;

                    if (_websiteItemsView)
                    {
                        return IsTypeWhiteListed(type);
                    }

                    return true;
                };
                _dataGroupingProviderHelper.OnGetPayload = token => null;

            }
        }
        


        /// <exclude />
        public bool ContainsLocalizedData
        {
            get
            {
                foreach (Type type in DataFacade.GetGeneratedInterfaces())
                {
                    if (DataLocalizationFacade.IsLocalized(type))
                    {
                        return true;
                    }
                }

                return false;
            }
        }



        /// <exclude />
        public IEnumerable<Element> GetRoots(SearchToken seachToken)
        {
            List<Element> roots = new List<Element>();

            Element globalDataElement;
            if (_websiteItemsView)
            {
                globalDataElement = new Element(_providerContext.CreateElementHandle(new GeneratedDataTypesElementProviderRootEntityToken(_providerContext.ProviderName, GeneratedDataTypesElementProviderRootEntityToken.GlobalDataTypeFolderId)))
                {
                    VisualData = new ElementVisualizedData
                    {
                        Label = Texts.GlobalDataFolderLabel_OnlyGlobalData,
                        ToolTip = Texts.GlobalDataFolderToolTip_OnlyGlobalData,
                        HasChildren = true,
                        Icon = GeneratedDataTypesElementProvider.InterfaceClosed,
                        OpenedIcon = GeneratedDataTypesElementProvider.InterfaceOpen
                    }
                };
            }
            else
            {
                globalDataElement = new Element(_providerContext.CreateElementHandle(new GeneratedDataTypesElementProviderRootEntityToken(_providerContext.ProviderName, GeneratedDataTypesElementProviderRootEntityToken.GlobalDataTypeFolderId)))
                    {
                        VisualData = new ElementVisualizedData
                        {
                            Label = Texts.GlobalDataFolderLabel,
                            ToolTip = Texts.GlobalDataFolderToolTip,
                            HasChildren = GlobalDataTypeFacade.GetAllGlobalDataTypes().Any(),
                            Icon = GeneratedDataTypesElementProvider.RootClosed,
                            OpenedIcon = GeneratedDataTypesElementProvider.RootOpen
                        }
                    };
            }

            if (!_websiteItemsView)
            {
                globalDataElement.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.AddNewInterfaceTypeWorkflow"), _addNewInterfaceTypePermissionTypes) { Payload = _providerContext.ProviderName }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("Add"),
                            ToolTip = GetText("AddToolTip"),
                            Icon = GeneratedDataTypesElementProvider.AddDataTypeIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });
            }

            globalDataElement.AddAction(
                new ElementAction(new ActionHandle(new ViewUnpublishedItemsActionToken()))
                {
                    VisualData = new ActionVisualizedData
                    {
                        Label = GetText("ViewUnpublishedItems"),
                        ToolTip = GetText("ViewUnpublishedItemsToolTip"),
                        Icon = GeneratedDataTypesElementProvider.ListUnpublishedItemsIcon,
                        Disabled = false,
                        ActionLocation = new ActionLocation
                        {
                            ActionType = ActionType.Other,
                            IsInFolder = false,
                            IsInToolbar = true,
                            ActionGroup = ViewActionGroup
                        }
                    }
                });


            roots.Add(globalDataElement);


            if (!_websiteItemsView)
            {
                bool staticDataTypesFolderHasChildren = DataFacade.GetAllKnownInterfaces().Any(t => t.IsStaticDataType());

                Element staticDataTypesFolderElement = new Element(_providerContext.CreateElementHandle(new GeneratedDataTypesElementProviderRootEntityToken(_providerContext.ProviderName, GeneratedDataTypesElementProviderRootEntityToken.StaticGlobalDataTypeFolderId)))
                {
                    VisualData = new ElementVisualizedData
                    {
                        Label = Texts.StaticDataTypesFolderLabel,
                        ToolTip = Texts.StaticDataTypesFolderToolTip, 
                        HasChildren = staticDataTypesFolderHasChildren,
                        Icon = GeneratedDataTypesElementProvider.RootClosed,
                        OpenedIcon = GeneratedDataTypesElementProvider.RootOpen
                    }
                };

                roots.Add(staticDataTypesFolderElement);

                bool pageDataFolderHasChildren = PageFolderFacade.GetAllFolderTypes().Any();

                Element pageDataFolderElement = new Element(_providerContext.CreateElementHandle(new GeneratedDataTypesElementProviderRootEntityToken(_providerContext.ProviderName, GeneratedDataTypesElementProviderRootEntityToken.PageDataFolderTypeFolderId)))
                {
                    VisualData = new ElementVisualizedData
                    {
                        Label = GetText("PageDataFolderDataFolderLabel"),
                        ToolTip = GetText("PageDataFolderDataFolderToolTip"),
                        HasChildren = pageDataFolderHasChildren,
                        Icon = GeneratedDataTypesElementProvider.RootClosed,
                        OpenedIcon = GeneratedDataTypesElementProvider.RootOpen
                    }
                };

                pageDataFolderElement.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.AddNewAggregationTypeWorkflow"), _addNewInterfaceTypePermissionTypes) { Payload = TypeManager.SerializeType(typeof(IPage)) }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("AddDataFolder"),
                            ToolTip = GetText("AddDataFolderToolTip"),
                            Icon = GeneratedDataTypesElementProvider.AddDataTypeIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

                roots.Add(pageDataFolderElement);


                bool pageMetaDataHasChildren = PageMetaDataFacade.GetAllMetaDataTypes().Any();


                Element pageMetaDataElement = new Element(_providerContext.CreateElementHandle(new GeneratedDataTypesElementProviderRootEntityToken(_providerContext.ProviderName, GeneratedDataTypesElementProviderRootEntityToken.PageMetaDataTypeFolderId)))
                {
                    VisualData = new ElementVisualizedData
                    {
                        Label = GetText("PageMetaDataFolderLabel"),
                        ToolTip = GetText("PageMetaDataFolderToolTip"),
                        HasChildren = pageMetaDataHasChildren,
                        Icon = GeneratedDataTypesElementProvider.RootClosed,
                        OpenedIcon = GeneratedDataTypesElementProvider.RootOpen
                    }
                };

                pageMetaDataElement.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.AddNewCompositionTypeWorkflow"), _addNewInterfaceTypePermissionTypes) { Payload = TypeManager.SerializeType(typeof(IPage)) }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("AddMetaDataLabel"),
                            ToolTip = GetText("AddMetaDataToolTip"),
                            Icon = GeneratedDataTypesElementProvider.AddDataTypeIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

                roots.Add(pageMetaDataElement);
            }


            return roots;
        }



        /// <exclude />
        public IEnumerable<Element> GetForeignRoots(SearchToken searchToken)
        {
            return GetRoots(searchToken);
        }



        /// <exclude />
        public IEnumerable<Element> GetChildren(EntityToken entityToken, SearchToken searchToken)
        {
            return GetChildren(entityToken, searchToken, false);
        }



        /// <exclude />
        public IEnumerable<Element> GetForeignChildren(EntityToken entityToken, SearchToken searchToken)
        {
            return GetChildren(entityToken, searchToken, true);
        }



        private IEnumerable<Element> GetChildren(EntityToken entityToken, SearchToken searchToken, bool showForeignChildren)
        {
            if (entityToken is GeneratedDataTypesElementProviderRootEntityToken)
            {
                return GetRootChildren(searchToken, entityToken as GeneratedDataTypesElementProviderRootEntityToken);
            }

            if (entityToken is GeneratedDataTypesElementProviderTypeEntityToken)
            {
                var castedEntityToken = entityToken as GeneratedDataTypesElementProviderTypeEntityToken;

                string typeManagerName = castedEntityToken.SerializedTypeName;

                Type type = TypeManager.TryGetType(typeManagerName);

                if (type != null)
                {
                    // These are never shown in the tree
                    if (typeof(IPageMetaData).IsAssignableFrom(type))
                    {
                        return new List<Element>();
                    }

                    IEnumerable<Element> elements = _dataGroupingProviderHelper.GetRootGroupFolders(type, entityToken, showForeignChildren);

                    return elements.ToList();
                }

                Log.LogWarning("GeneratedDataTypesElementProvider", "Can not get children for unknown type '{0}'".FormatWith(typeManagerName));

                return null;
            }

            if (entityToken is DataGroupingProviderHelperEntityToken)
            {
                List<Element> elements = _dataGroupingProviderHelper.GetGroupChildren(entityToken as DataGroupingProviderHelperEntityToken, showForeignChildren).ToList();

                return elements;
            }

            if (entityToken is DataEntityToken)
            {
                return new List<Element>();
            }

            throw new InvalidOperationException("This code should not be reachable");
        }



        private List<Element> GetRootChildren(SearchToken searchToken, GeneratedDataTypesElementProviderRootEntityToken entityToken)
        {
            if (entityToken.Id == GeneratedDataTypesElementProviderRootEntityToken.GlobalDataTypeFolderId)
            {
                return GetGlobalDataTypesElements(searchToken, false).OrderBy(f => f.VisualData.Label).ToList();
            }
            if (entityToken.Id == GeneratedDataTypesElementProviderRootEntityToken.StaticGlobalDataTypeFolderId)
            {
                return GetGlobalDataTypesElements(searchToken, true).OrderBy(f => f.VisualData.Label).ToList();
            }
            if (entityToken.Id == GeneratedDataTypesElementProviderRootEntityToken.PageDataFolderTypeFolderId)
            {
                return GetPageFolderDataTypesElements(searchToken).OrderBy(f => f.VisualData.Label).ToList();
            }
            if (entityToken.Id == GeneratedDataTypesElementProviderRootEntityToken.PageMetaDataTypeFolderId)
            {
                return GetPageMetaDataTypesElements(searchToken).OrderBy(f => f.VisualData.Label).ToList();
            }

            throw new InvalidOperationException("This code should not be reachable");
        }



        private List<Element> GetGlobalDataTypesElements(SearchToken searchToken, bool showStatic)
        {
            List<Element> elements = new List<Element>();

            IEnumerable<Type> interfaceList;
            
            if (_websiteItemsView)
            {
                // Showing both generated and custom static interfaces
                interfaceList = DataFacade.GetAllInterfaces().Where(i => i.IsStaticDataType() || i.IsGenerated());
            }
            else
            {
                if (showStatic)
                {
                    interfaceList = DataFacade.GetAllInterfaces().Where(t => t.IsStaticDataType());
                }
                else
                {
                    interfaceList = DataFacade.GetGeneratedInterfaces();
                }
            }

            interfaceList = interfaceList.OrderBy(t => t.FullName);

            interfaceList = interfaceList.Except(PageFolderFacade.GetAllFolderTypes());
            interfaceList = interfaceList.Except(PageMetaDataFacade.GetAllMetaDataTypes());

            if (searchToken.IsValidKeyword())
            {
                interfaceList = interfaceList.Where(x => x.FullName.ToLower().Contains(searchToken.Keyword.ToLower())).ToList();
            }


            Dictionary<Type, DataTypeDescriptor> interfaces = interfaceList.ToDictionary(f => f, DynamicTypeManager.GetDataTypeDescriptor);

            IEnumerable<KeyValuePair<Type, DataTypeDescriptor>> sortedInterfaces = interfaces;
            if (_websiteItemsView)
            {
                sortedInterfaces = interfaces.OrderBy(f => f.Value.Title);
            }

            List<string> whiteList = DataFacade.GetData<IGeneratedTypeWhiteList>().Select(element => element.TypeManagerTypeName).ToList();

            foreach (var kvp in sortedInterfaces)
            {
                Type type = kvp.Key;
                DataTypeDescriptor dataTypeDescriptor = kvp.Value;

                DataTypeDescriptor tempDescriptor;

                bool storeCreated = DynamicTypeManager.TryGetDataTypeDescriptor(type.GetImmutableTypeId(), out tempDescriptor);

                string typeName = TypeManager.SerializeType(type);

                if (_websiteItemsView && !whiteList.Contains(typeName))
                {
                    continue;
                }

                DataScopeIdentifier dataScopeIdentifier = DataScopeIdentifier.Public;
                if (typeof(IPublishControlled).IsAssignableFrom(type))
                {
                    dataScopeIdentifier = DataScopeIdentifier.Administrated;
                }

                Exception queryDataException = null;

                bool hasChildren = false;
                try
                {
                    using (new DataScope(dataScopeIdentifier))
                    {
                        hasChildren = storeCreated && DataFacade.GetData(type).Any();

                        if (!hasChildren && storeCreated && DataLocalizationFacade.IsLocalized(type))
                        {
                            using (new DataScope(UserSettings.ForeignLocaleCultureInfo))
                            {
                                hasChildren |= DataFacade.GetData(type).Any();
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Log.LogError(LogTitle, ex);
                    queryDataException = ex;
                }
                
                string label = type.FullName;
                if (_websiteItemsView)
                {
                    label = dataTypeDescriptor.Title;
                }

                bool failedToLoad = queryDataException != null;

                Element element = new Element(_providerContext.CreateElementHandle(
                    new GeneratedDataTypesElementProviderTypeEntityToken(typeName, _providerContext.ProviderName,
                        showStatic ? GeneratedDataTypesElementProviderRootEntityToken.StaticGlobalDataTypeFolderId : GeneratedDataTypesElementProviderRootEntityToken.GlobalDataTypeFolderId)))
                {
                    VisualData = new ElementVisualizedData
                    {
                        Label = label,
                        ToolTip = !failedToLoad ? label : GetNestedExceptionMessage(queryDataException),
                        HasChildren = hasChildren,
                        Icon = !failedToLoad ? GeneratedDataTypesElementProvider.InterfaceClosed : ErrorIcon,
                        OpenedIcon = !failedToLoad ? GeneratedDataTypesElementProvider.InterfaceOpen : ErrorIcon
                    }
                };


                if (storeCreated && !_websiteItemsView)
                {
                    AddNonShowOnlyGlobalActions(type, typeName, element);
                }

                if (storeCreated)
                {
                    element.AddAction(new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.AddNewDataWorkflow"), _addNewDataPermissionTypes) { DoIgnoreEntityTokenLocking = true }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("AddData"),
                            ToolTip = GetText("AddDataToolTip"),
                            Icon = GeneratedDataTypesElementProvider.AddDataIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Add,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });
                }

                if (RuntimeInformation.IsDebugBuild)
                {
                    element.AddAction(
                    new ElementAction(new ActionHandle(new DataTypeDescriptorToXmlActionToken()))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("ToXmlLabel"),
                            ToolTip = GetText("ToXmlToolTip"),
                            Icon = GeneratedDataTypesElementProvider.ToXmlIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Add,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = AppendedActionGroup
                            }
                        }
                    });
                }

                // TODO: add "Create store" action
                elements.Add(element);
            }

            return elements;
        }

        private string GetNestedExceptionMessage(Exception queryDataException)
        {
            var ex = queryDataException;

            while (ex is TargetInvocationException)
            {
                ex = ex.InnerException;
            }

            return ex.Message;
        }

        
        private void AddNonShowOnlyGlobalActions(Type type, string typeName, Element element)
        {
            DataTypeDescriptor dataTypeDescriptor = DynamicTypeManager.GetDataTypeDescriptor(type.GetImmutableTypeId());
            bool isEditable = dataTypeDescriptor.IsCodeGenerated;

            if (DataLocalizationFacade.UseLocalization)
            {
                element.AddAction(GetChangeLocalizationElementAction(type, isEditable));
            }

            element.AddAction(
                new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EditInterfaceTypeWorkflow"), _editInterfaceTypePermissionTypes) { Payload = _providerContext.ProviderName }))
                {
                    VisualData = new ActionVisualizedData
                    {
                        Label = GetText("Edit"),
                        ToolTip = GetText("EditToolTip"),
                        Icon = GeneratedDataTypesElementProvider.EditDataTypeIcon,
                        Disabled = !isEditable,
                        ActionLocation = new ActionLocation
                        {
                            ActionType = ActionType.Edit,
                            IsInFolder = false,
                            IsInToolbar = true,
                            ActionGroup = PrimaryActionGroup
                        }
                    }
                });

            element.AddAction(
                new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.DeleteInterfaceTypeWorkflow"), _deleteInterfaceTypePermissionTypes) { Payload = _providerContext.ProviderName }))
                {
                    VisualData = new ActionVisualizedData
                    {
                        Label = GetText("Delete"),
                        ToolTip = GetText("DeleteToolTip"),
                        Icon = GeneratedDataTypesElementProvider.DeleteDataTypeIcon,
                        Disabled = !isEditable,
                        ActionLocation = new ActionLocation
                        {
                            ActionType = ActionType.Delete,
                            IsInFolder = false,
                            IsInToolbar = true,
                            ActionGroup = PrimaryActionGroup
                        }
                    }
                });

            element.AddAction(
                new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EditFormWorkflow"), _editFormMarkupPermissionTypes) { Payload = _providerContext.ProviderName, DoIgnoreEntityTokenLocking = true }))
                {
                    VisualData = new ActionVisualizedData
                    {
                        Label = GetText("EditFormMarkup"),
                        ToolTip = GetText("EditFormMarkupToolTip"),
                        Icon = GeneratedDataTypesElementProvider.EditFormMarkupIcon,
                        Disabled = false,
                        ActionLocation = new ActionLocation
                        {
                            ActionType = ActionType.Edit,
                            IsInFolder = false,
                            IsInToolbar = true,
                            ActionGroup = PrimaryActionGroup
                        }
                    }
                });


            bool exists = DataFacade.GetData<IGeneratedTypeWhiteList>().Any(f => f.TypeManagerTypeName == typeName);


            element.AddAction(
                new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType(
                    exists == false ?
                        "Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.AddTypeToWhiteListWorkflow" :
                        "Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.RemoveTypeFromWhiteListWorkflow"
                    ), _addNewInterfaceTypePermissionTypes) { DoIgnoreEntityTokenLocking = true }))
                {
                    VisualData = new ActionVisualizedData
                    {
                        Label = GetText("ShowInContent"),
                        ToolTip = GetText("ShowInContentToolTip"),
                        Icon = GeneratedDataTypesElementProvider.ShowincontentareaIcon,
                        Disabled = false,
                        ActionCheckedStatus = exists == false ? ActionCheckedStatus.Unchecked : ActionCheckedStatus.Checked,
                        ActionLocation = new ActionLocation
                        {
                            ActionType = ActionType.Add,
                            IsInFolder = false,
                            IsInToolbar = false,
                            ActionGroup = PrimaryActionGroup
                        }
                    }
                });
        }

        private static ElementAction GetChangeLocalizationElementAction(Type type, bool isEditable)
        {
            if (DataLocalizationFacade.IsLocalized(type) == false)
            {
                return
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EnableTypeLocalizationWorkflow"), _editInterfaceTypePermissionTypes)))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("EnableLocalization"),
                            ToolTip = GetText("EnableLocalizationToolTip"),
                            Icon = GeneratedDataTypesElementProvider.LocalizeDataTypeIcon,
                            Disabled = DataLocalizationFacade.ActiveLocalizationCultures.Any() == false || !isEditable,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Other,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    };
            }


            return
                new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.DisableTypeLocalizationWorkflow"), _editInterfaceTypePermissionTypes)))
                {
                    VisualData = new ActionVisualizedData
                    {
                        Label = GetText("DisableLocalization"),
                        ToolTip = GetText("DisableLocalizationToolTip"),
                        Icon = GeneratedDataTypesElementProvider.DelocalizeDataTypeIcon,
                        Disabled = !isEditable,
                        ActionLocation = new ActionLocation
                        {
                            ActionType = ActionType.Other,
                            IsInFolder = false,
                            IsInToolbar = true,
                            ActionGroup = PrimaryActionGroup
                        }
                    }
                };
        }
        

        private List<Element> GetPageFolderDataTypesElements(SearchToken searchToken)
        {
            List<Element> elements = new List<Element>();

            IEnumerable<Type> types = PageFolderFacade.GetAllFolderTypes();

            if (searchToken.IsValidKeyword())
            {
                types = types.Where(x => x.FullName.ToLower().Contains(searchToken.Keyword.ToLower()));
            }

            foreach (Type type in types)
            {
                DataTypeDescriptor dataTypeDescriptor = DynamicTypeManager.GetDataTypeDescriptor(type.GetImmutableTypeId());
                bool isEditable = dataTypeDescriptor.IsCodeGenerated;

                string typeName = TypeManager.SerializeType(type);

                bool hasChildren = false;

                Element element = new Element(_providerContext.CreateElementHandle(new GeneratedDataTypesElementProviderTypeEntityToken(typeName, _providerContext.ProviderName, GeneratedDataTypesElementProviderRootEntityToken.PageDataFolderTypeFolderId)))
                {
                    VisualData = new ElementVisualizedData
                    {
                        Label = type.FullName,
                        ToolTip = type.FullName,
                        HasChildren = hasChildren,
                        Icon = GeneratedDataTypesElementProvider.InterfaceClosed,
                        OpenedIcon = GeneratedDataTypesElementProvider.InterfaceOpen
                    }
                };

                if (DataLocalizationFacade.UseLocalization)
                {
                    element.AddAction(GetChangeLocalizationElementAction(type, true));
                }


                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EditAggregationTypeWorkflow"), _editInterfaceTypePermissionTypes) { Payload = typeName }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("EditDataFolderTypeLabel"),
                            ToolTip = GetText("EditDataFolderTypeToolTip"),
                            Icon = GeneratedDataTypesElementProvider.EditDataTypeIcon,
                            Disabled = !isEditable,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.DeleteAggregationTypeWorkflow"), _deleteInterfaceTypePermissionTypes) { Payload = typeName }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("DeleteDataFolderTypeLabel"),
                            ToolTip = GetText("DeleteDataFolderTypeToolTip"),
                            Icon = GeneratedDataTypesElementProvider.DeleteDataTypeIcon,
                            Disabled = !isEditable,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Delete,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });


                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EditFormWorkflow"), _editFormMarkupPermissionTypes) { Payload = _providerContext.ProviderName, DoIgnoreEntityTokenLocking = true }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("EditFormMarkup"),
                            ToolTip = GetText("EditFormMarkupToolTip"),
                            Icon = GeneratedDataTypesElementProvider.EditFormMarkupIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

                elements.Add(element);
            }

            return elements;
        }



        private List<Element> GetPageMetaDataTypesElements(SearchToken searchToken)
        {
            List<Element> elements = new List<Element>();

            IEnumerable<Type> types = PageMetaDataFacade.GetAllMetaDataTypes();

            if (searchToken.IsValidKeyword())
            {
                types = types.Where(x => x.FullName.ToLower().Contains(searchToken.Keyword.ToLower()));
            }

            foreach (Type type in types)
            {
                DataTypeDescriptor dataTypeDescriptor = DynamicTypeManager.GetDataTypeDescriptor(type.GetImmutableTypeId());
                bool isEditable = dataTypeDescriptor.IsCodeGenerated;

                string typeName = TypeManager.SerializeType(type);

                bool hasChildren = false;

                Element element = new Element(_providerContext.CreateElementHandle(new GeneratedDataTypesElementProviderTypeEntityToken(typeName, _providerContext.ProviderName, GeneratedDataTypesElementProviderRootEntityToken.PageMetaDataTypeFolderId)))
                {
                    VisualData = new ElementVisualizedData
                    {
                        Label = type.FullName,
                        ToolTip = type.FullName,
                        HasChildren = hasChildren,
                        Icon = GeneratedDataTypesElementProvider.InterfaceClosed,
                        OpenedIcon = GeneratedDataTypesElementProvider.InterfaceOpen
                    }
                };


                if (DataLocalizationFacade.UseLocalization)
                {
                    element.AddAction(GetChangeLocalizationElementAction(type, isEditable));
                }


                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EditCompositionTypeWorkflow"), _editInterfaceTypePermissionTypes) { Payload = typeName }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("EditMetaDataTypeLabel"),
                            ToolTip = GetText("EditMetaDataTypeToolTip"),
                            Icon = GeneratedDataTypesElementProvider.EditDataTypeIcon,
                            Disabled = !isEditable,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.DeleteCompositionTypeWorkflow"), _deleteInterfaceTypePermissionTypes) { Payload = typeName }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("DeleteMetaDataTypeLabel"),
                            ToolTip = GetText("DeleteMetaDataTypeToolTip"),
                            Icon = GeneratedDataTypesElementProvider.DeleteDataTypeIcon,
                            Disabled = !isEditable,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Delete,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EditFormWorkflow"), _editFormMarkupPermissionTypes) { DoIgnoreEntityTokenLocking = true }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("EditFormMarkup"),
                            ToolTip = GetText("EditFormMarkupToolTip"),
                            Icon = GeneratedDataTypesElementProvider.EditFormMarkupIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

                elements.Add(element);
            }

            return elements;
        }



        private Element GetElementFromData(IData data)
        {
            Type type = data.DataSourceId.InterfaceType;

            string label = data.GetLabel(true);

            Element element = new Element(_providerContext.CreateElementHandle(data.GetDataEntityToken()))
            {
                VisualData = new ElementVisualizedData
                {
                    Label = label,
                    ToolTip = label,
                    HasChildren = false,
                    Icon = (data is IPublishControlled ? DataIconLookup[((IPublishControlled)data).PublicationStatus] : DataIconFacade.DataPublishedIcon)
                }
            };



            if (PageMetaDataFacade.GetAllMetaDataTypes().Contains(type) == false)
            {
                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.EditDataWorkflow"), _editDataPermissionTypes)))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("EditData"),
                            ToolTip = GetText("EditDataToolTip"),
                            Icon = GeneratedDataTypesElementProvider.EditDataIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Edit,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });


                element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.DeleteDataWorkflow"), _deleteDataPermissionTypes)))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("DeleteData"),
                            ToolTip = GetText("DeleteDataToolTip"),
                            Icon = GeneratedDataTypesElementProvider.DeleteDataIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Delete,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });
            }

            return element;
        }



        private Element GetGhostedElementFromData(IData data)
        {
            Type type = data.DataSourceId.InterfaceType;

            string label = string.Format("{0} ({1})", data.GetLabel(true), DataLocalizationFacade.GetCultureTitle(UserSettings.ForeignLocaleCultureInfo));

            Element element = new Element(_providerContext.CreateElementHandle(data.GetDataEntityToken()))
            {
                VisualData = new ElementVisualizedData
                {
                    Label = label,
                    ToolTip = label,
                    HasChildren = false,
                    Icon = DataIconFacade.DataGhostedIcon,
                    IsDisabled = false
                }
            };

            element.AddAction(new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.LocalizeDataWorkflow"), _localizeDataPermissionTypes) { Payload = "Global" }))
            {
                VisualData = new ActionVisualizedData
                {
                    Label = GetText("LocalizeData"),
                    ToolTip = GetText("LocalizeDataToolTip"),
                    Icon = GeneratedDataTypesElementProvider.LocalizeDataIcon,
                    Disabled = false,
                    ActionLocation = new ActionLocation
                    {
                        ActionType = ActionType.Add,
                        IsInFolder = false,
                        IsInToolbar = true,
                        ActionGroup = PrimaryActionGroup
                    }
                }
            });

            return element;
        }



        private Element GetDisabledElementFromData(IData data)
        {
            string label = string.Format("{0} ({1})", data.GetLabel(true), DataLocalizationFacade.GetCultureTitle(UserSettings.ForeignLocaleCultureInfo));

            Element element = new Element(_providerContext.CreateElementHandle(data.GetDataEntityToken()))
            {
                VisualData = new ElementVisualizedData
                {
                    Label = label,
                    ToolTip = GetText("DisabledData"),
                    HasChildren = false,
                    Icon = DataIconFacade.DataDisabledIcon,
                    IsDisabled = true
                }
            };

            return element;
        }



        private Element AddGroupFolderActions(Element element, PropertyInfoValueCollection propertyInfoValueCollection)
        {
            StringBuilder sb = new StringBuilder();
            foreach (var kvp in propertyInfoValueCollection.PropertyValues)
            {
                string value = ValueTypeConverter.Convert<string>(kvp.Value);
                StringConversionServices.SerializeKeyValuePair<string>(sb, kvp.Key.Name, value);
            }

            element.AddAction(
                    new ElementAction(new ActionHandle(new WorkflowActionToken(WorkflowFacade.GetWorkflowType("Composite.Plugins.Elements.ElementProviders.GeneratedDataTypesElementProvider.AddNewDataWorkflow"), _addNewDataPermissionTypes) { DoIgnoreEntityTokenLocking = true, Payload = sb.ToString() }))
                    {
                        VisualData = new ActionVisualizedData
                        {
                            Label = GetText("AddData"),
                            ToolTip = GetText("AddDataToolTip"),
                            Icon = GeneratedDataTypesElementProvider.AddDataIcon,
                            Disabled = false,
                            ActionLocation = new ActionLocation
                            {
                                ActionType = ActionType.Add,
                                IsInFolder = false,
                                IsInToolbar = true,
                                ActionGroup = PrimaryActionGroup
                            }
                        }
                    });

            return element;
        }


        private static bool IsTypeWhiteListed(Type type)
        {
            string typeManagerTypeName = TypeManager.SerializeType(type);

            IEnumerable<IGeneratedTypeWhiteList> whileList = DataFacade.GetData<IGeneratedTypeWhiteList>(true);

            bool isWhiteListed = whileList.Where(f => f.TypeManagerTypeName == typeManagerTypeName).Any();

            return isWhiteListed;
        }



        private EntityToken GetRootParentEntityToken(Type type, EntityToken entityToken)
        {
            bool isPageFolder = PageFolderFacade.GetAllFolderTypes().Contains(type);

            if (!isPageFolder)
            {
                if (_websiteItemsView && !IsTypeWhiteListed(type)) return null;

                return new GeneratedDataTypesElementProviderTypeEntityToken(
                    TypeManager.SerializeType(type),
                    _providerContext.ProviderName,
                    (_websiteItemsView || type.IsGenerated())
                    ? GeneratedDataTypesElementProviderRootEntityToken.GlobalDataTypeFolderId
                    : GeneratedDataTypesElementProviderRootEntityToken.StaticGlobalDataTypeFolderId
                );
            }

            if (_websiteItemsView) return null;

            var groupingEntityToken = entityToken as DataGroupingProviderHelperEntityToken;
            if (groupingEntityToken != null && !groupingEntityToken.Payload.IsNullOrEmpty())
            {
                // Grouping entity tokens with payload aren't attached to the data type folder in the 'Data' perspective
                return null;
            }

            return new GeneratedDataTypesElementProviderTypeEntityToken(
                    TypeManager.SerializeType(type),
                    _providerContext.ProviderName,
                    GeneratedDataTypesElementProviderRootEntityToken.PageDataFolderTypeFolderId
                );
        }



        private static ResourceHandle GetIconHandle(string name)
        {
            return new ResourceHandle(BuildInIconProviderName.ProviderName, name);
        }

        internal static string GetText(string key)
        {
            return StringResourceSystemFacade.GetString("Composite.Plugins.GeneratedDataTypesElementProvider", key);
        }
    }


    [Assembler(typeof(GeneratedDataTypesElementProviderAssembler))]
    internal sealed class GeneratedDataTypesElementProviderData : HooklessElementProviderData
    {
        private const string _onlyShowGlobalDatasPropertyName = "onlyShowGlobalDatas";
        /// <exclude />
        [ConfigurationProperty(_onlyShowGlobalDatasPropertyName, IsRequired = false, DefaultValue = false)]
        public bool OnlyShowGlobalDatas
        {
            get { return (bool)base[_onlyShowGlobalDatasPropertyName]; }
            set { base[_onlyShowGlobalDatasPropertyName] = value; }
        }
    }



    internal sealed class GeneratedDataTypesElementProviderAssembler : IAssembler<IHooklessElementProvider, HooklessElementProviderData>
    {
        public IHooklessElementProvider Assemble(IBuilderContext context, HooklessElementProviderData objectConfiguration, IConfigurationSource configurationSource, ConfigurationReflectionCache reflectionCache)
        {
            GeneratedDataTypesElementProviderData data = (GeneratedDataTypesElementProviderData)objectConfiguration;

            return new GeneratedDataTypesElementProvider(data.OnlyShowGlobalDatas);
        }
    }
}
