/*

The contents of this web application are subject to the Mozilla Public License Version 1.1 (the "License"); you may not use this web application except in compliance with the License. 
You may obtain a copy of the License at http://www.mozilla.org/MPL/.

Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
See the License for the specific language governing rights and limitations under the License.

The Original Code is owned by and the Initial Developer of the Original Code is Composite A/S (Danish business reg.no. 21744409). All Rights Reserved

Section 11 of the License is EXPRESSLY amended to include a provision stating that any dispute, including but not limited to disputes related to the enforcement of the License, to which Composite A/S as owner of the Original Code, as Initial Developer or in any other role, becomes a part to shall be governed by Danish law and be initiated before the Copenhagen City Court ("K�benhavns Byret")
                        
*/

using System.Collections.Generic;
using Composite.C1Console.Elements;
using Composite.C1Console.Elements.Plugins.ElementProvider;
using Composite.Core.ResourceSystem;
using Composite.Core.ResourceSystem.Icons;
using Composite.C1Console.Security;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration.ObjectBuilder;


namespace Composite.Plugins.Elements.ElementProviders.LogElementProvider
{
    [ConfigurationElementType(typeof(LogElementProviderData))]
    internal sealed class LogElementProvider : IHooklessElementProvider
    {
        private ElementProviderContext _elementProviderContext;


        public static ResourceHandle RootFolderIcon { get { return GetIconHandle("log-folder-closed"); } }
        public static ResourceHandle ShowLogIcon { get { return GetIconHandle("log-showlog"); } }
        private static ResourceHandle GetIconHandle(string name) { return new ResourceHandle(BuildInIconProviderName.ProviderName, name); }

        private static readonly ActionGroup PrimaryActionGroup = new ActionGroup(ActionGroupPriority.PrimaryHigh);

        public ElementProviderContext Context
        {
            set { _elementProviderContext = value; }
        }



        public IEnumerable<Element> GetRoots(SearchToken seachToken)
        {
            Element element = new Element(_elementProviderContext.CreateElementHandle(new LogElementProviderEntityToken(LogElementProviderEntityToken.RootFolderId)))
            {
                VisualData = new ElementVisualizedData
                {
                    Label = StringResourceSystemFacade.GetString("Composite.Management", "LogElementProvider.RootLabel"),
                    ToolTip = StringResourceSystemFacade.GetString("Composite.Management", "LogElementProvider.RootLabelToolTip"),
                    HasChildren = false,
                    Icon = LogElementProvider.RootFolderIcon,
                    OpenedIcon = LogElementProvider.RootFolderIcon
                }
            };



            element.AddAction(new ElementAction(new ActionHandle(new ShowLogActionToken()))
            {
                VisualData = new ActionVisualizedData
                {
                    Label = StringResourceSystemFacade.GetString("Composite.Management", "LogElementProvider.ShowLogLabel"),
                    ToolTip = StringResourceSystemFacade.GetString("Composite.Management", "LogElementProvider.ShowLogToolTip"),
                    Icon = LogElementProvider.ShowLogIcon,
                    Disabled = false,
                    ActionLocation = new ActionLocation
                    {
                        ActionType = ActionType.Other,
                        IsInFolder = false,
                        IsInToolbar = true,
                        ActionGroup = PrimaryActionGroup
                    }
                }
            });


            return new List<Element> { element };
        }



        public IEnumerable<Element> GetChildren(EntityToken entityToken, SearchToken seachToken)
        {
            return new List<Element>();
        }
    }




    [Assembler(typeof(NonConfigurableHooklessElementProviderAssembler))]
    internal sealed class LogElementProviderData : HooklessElementProviderData
    {
    }
}
