/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

/**
 * Composite link plugin.
 */
new function () {
	
	var URL_LINK = "${tiny}/plugins/compositelink/link.aspx";
	
	tinymce.create ( "tinymce.plugins.CompositeLinkPlugin", {
		
		getInfo : function() {
			return {
				longname : "Composite Link",
				author : "Composite A/S",
				authorurl : "http://www.composite.net",
				infourl : null,
				version : tinymce.majorVersion + "." + tinymce.minorVersion
			};
		},
		
		/**
		 * @param {tinymce.Editor} ed
		 * @param {string} url
		 */
		init : function ( ed, url ) {

			this.editor = ed;

			ed.addShortcut('ctrl+k', '', 'compositeInsertLink');  
		},
		
		/**
		 * @param {HTMLAnchorElement} a
		 * @param {tinymce.Editor} editor
		 * @param {DataBindingResultMap} result
		 */
		_insertLink : function ( anchor, editor, result ) {
			
			//editor.getDoc().execCommand ( "unlink", false, null );

			editor.execCommand('mceInsertLink', false, {
				href: result.get("href"),
				rel: result.get("rel"),
				title: result.get("title"),
				tinymcetargetalias: result.get("blank") ? "_blank" : "",
				id: result.get( "id" ),
				"class": result.get("classname")
			});
		},
		
		
		/**
		 * @param {string} cmd
		 * @param {boolean} ui
		 * @param {string} value
		 */
		execCommand : function ( cmd, ui, value ) {
			
			var result = false;
			var self = this;
			var editor = this.editor;
			var editorBinding = editor.theme.editorBinding;
			
			if ( cmd == "unlink" ) {
				
				setTimeout ( function () {
					editorBinding.checkForDirty ();
					editorBinding.blurEditor ();
				}, 50 );
				// result is still false - relay to higher authority
				
			} else if ( cmd == "compositeInsertLink" ) {
				
				var elm = editor.selection.getNode ();
				var anchor = editor.dom.getParent ( elm, "a" );
				
				if ( value == null ) {
					value = anchor != null ? "update" : "insert";
				}
				 
				editorBinding.enableDialogMode ();
				
				var dialogHandler = {
					handleDialogResponse : function ( response, result ) {
					
						editorBinding.disableDialogMode ();
						
						if ( response == Dialog.RESPONSE_ACCEPT ) {		
							self._insertLink ( anchor, editor, result );
							editorBinding.getButtonForCommand ( "compositeInsertLink" ).disable ();
							editorBinding.getButtonForCommand ( "unlink" ).disable ();
							editorBinding.checkForDirty ();
							editorBinding.blurEditor ();
						}
					}
				};
				
				Dialog.invokeModal ( 
					URL_LINK,
					dialogHandler, 
					{
						tinyAction 		: value,
						tinyWindow 		: window,
						tinyElement 	: anchor,
						tinyEngine 		: tinymce.EditorManager,
						tinyInstance 	: editor,
						tinyTheme 		: editor.theme,
						editorBinding 	: editor.theme.editorBinding
					}
				);
				
				result = true;
			} 
			return result;
		}
	});

	// Register plugin
	tinymce.PluginManager.add("compositelink", tinymce.plugins.CompositeLinkPlugin);
};