/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

/**
 * @class
 * Don't instantiate this class manually. Access through 
 * instance variable "Client" declared below. This 
 * instance should be considered a singleton class.
 */
function _Client () {
	
	var agent = navigator.userAgent.toLowerCase ();
	var platform = navigator.platform.toLowerCase ();

	var isExplorer = navigator.appName == "Microsoft Internet Explorer"; // IE<=10 OR IE11
	var isMozilla = !isExplorer && typeof document.createTreeWalker != "undefined" ;
	var isPrism = isMozilla && ( agent.indexOf ( "webrunner" ) >-1 || agent.indexOf ( "prism" ) >-1 );
	var hasTransitions = history.pushState != null;

	this.isMozilla = isMozilla;
	this.isFirefox = agent.indexOf("firefox") > -1;
	this.isWebKit = agent.indexOf("webkit") > -1;
	this.isExplorer = isExplorer;
	this.isExplorer6 = this.isExplorer && ( agent.indexOf ( "msie 6.0" ) > -1 || agent.indexOf ( "msie 6.1" ) > -1 );
	this.isExplorer8 = this.isExplorer && window.XDomainRequest != null;
	this.isExplorer11 = !!navigator.userAgent.match(/Trident\/7\./); 
	this.isPrism = isPrism;
	this.isWindows = platform.indexOf ( "win" ) > -1;
	this.isVista = this.isWindows && agent.indexOf ( "windows nt 6" ) > -1;
	this.isiPad = navigator.userAgent.match(/iPad/i) != null;
	
	var version = this._getFlashVersion ();
	this.hasFlash = ( version && version >= 9 );
	this.hasTransitions = hasTransitions;

	this.canvas = !!document.createElement('canvas').getContext;

	this.hasSpellcheck = this.isFirefox || this.isExplorer && document.documentElement.spellcheck;
	this.hasXSLTProcessor = this.isMozilla && !this.isExplorer11;

	return this;
}
/*
 * Public fields.
 */
_Client.prototype = {

	/** 
	* Is Internet Explorer?
	* @type {boolean} 
	*/
	isExplorer: false,

	/**
	* Is Gecko derivate? 
	* @type {boolean} 
	*/
	isMozilla: false,

	/** 
	* True for Mozilla Prism.
	* @type {boolean} 
	*/
	isPrism: false,

	/**
	* Has Flash version 10 minimum? 
	* @type {boolean} 
	*/
	hasFlash: false,

	/**
	* Is Microsoft Windows? Macintosh and Linux 
	* distros uniformely treated as not-Windows.
	* @type {boolean}
	*/
	isWindows: false,

	/**
	* Is Vista or Windows 7? As opposed to Windows XP.
	* @type {boolean}
	*/
	isVista: false,

	/**
	* Supports CSS transitions?
	* @type {boolean}
	*/
	hasTransitions: false,

	/**
	* Get Flash version.
	* @return {int}
	*/
	_getFlashVersion: function () {

		var result = null;
		var maxversion = 10; // maximum version tested for

		// detect flash version
		try {
			if (this.isMozilla == true) {
				if (typeof navigator.plugins["Shockwave Flash"] != "undefined") {
					var plugin = navigator.plugins["Shockwave Flash"];
					if (plugin) {
						var desc = plugin.description;
						if (desc != null) {
							result = desc.charAt(desc.indexOf(".") - 1);
						}
					}
				}
			} else {
				for (var i = 2; i <= maxversion; i++) {
					try {
						new ActiveXObject("ShockwaveFlash.ShockwaveFlash." + i);
						result = i;
					} catch (exception) {
						continue;
					}
				}
			}
		}
		catch (exception) { };
		return result;
	},

	/**
	* Client qualified for the awesome Composite C1 experience?
	* @return {boolean}
	*/
	qualifies: function () {

		var result = true;
		var isOldFox = false;
		if (this.isMozilla && !this.isWebKit && !this.isExplorer11) {
			isOldFox = (document.documentElement.mozMatchesSelector === undefined);
		}
		if (window.opera != null || isOldFox || this.isExplorer && !this.canvas) {
			result = false;
		}
		return result;
	},

	fixUI: function (html) {
		if (Client.isExplorer) {
			html = html.replace(/<ui:/g, "<").replace(/<\/ui:/g, "</");
			html = html.replace(/(<(\w+)[^>]*)\/>/g, "$1></$2>");
		}
		return html;
	}
}

/**
 * The instance that does it.
 * @type {_Client}
 */
var Client = new _Client ();
