/*

The contents of this web application are subject to the Mozilla Public License Version 1.1 (the "License"); you may not use this web application except in compliance with the License. 
You may obtain a copy of the License at http://www.mozilla.org/MPL/.

Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. 
See the License for the specific language governing rights and limitations under the License.

The Original Code is owned by and the Initial Developer of the Original Code is Composite A/S (Danish business reg.no. 21744409). All Rights Reserved

Section 11 of the License is EXPRESSLY amended to include a provision stating that any dispute, including but not limited to disputes related to the enforcement of the License, to which Composite A/S as owner of the Original Code, as Initial Developer or in any other role, becomes a part to shall be governed by Danish law and be initiated before the Copenhagen City Court ("K�benhavns Byret")
                        
*/

DockTabsBinding.prototype = new TabsBinding;
DockTabsBinding.prototype.constructor = DockTabsBinding;
DockTabsBinding.superclass = TabsBinding.prototype;
DockTabsBinding.NODENAME_TABBOX = "dock";
DockTabsBinding.TABBUTTON_IMPLEMENTATION = DockTabsButtonBinding;

/**
 * @class
 */
function DockTabsBinding () {

	/**
	 * @type {SystemLogger}
	 */
	this.logger = SystemLogger.getLogger ( "DockTabsBinding" );
}

/**
 * Identifies binding.
 */
DockTabsBinding.prototype.toString = function () {

	return "[DockTabsBinding]";
}


/**
 * @overloads {TabsBinding#buildDOMContent}
 */
DockTabsBinding.prototype.buildDOMContent = function () {

	DockTabsBinding.superclass.buildDOMContent.call ( this );
	
	if ( this.containingTabBoxBinding.type != DockBinding.TYPE_EXPLORER ) {
		this.controlGroupBinding = this.add (
			ControlGroupBinding.newInstance ( this.bindingDocument ) 
		);
		this.controlGroupBinding.attachClassName ( "docktabscontrolgroup" );
		// this.controlGroupBinding.add ( this.getControlBinding ( ControlBinding.TYPE_MINIMIZE ));
		this.controlGroupBinding.add ( this.getControlBinding ( ControlBinding.TYPE_MAXIMIZE ));
		this.controlGroupBinding.attachRecursive ();
	}
}

/**
 * Build DockControlBinding.
 * @param {string} type
 * @return {ControlBinding}
 */
DockTabsBinding.prototype.getControlBinding = function ( type ) {

	var controlBinding = DockControlBinding.newInstance ( this.bindingDocument );
	controlBinding.setControlType ( type );
	return controlBinding;
}

/**
 * @overloads {TabsBinding#flex}
 * @implements {IFlexible}
 */
DockTabsBinding.prototype.flex = function () {

	/*
	 * This fixes a horrendous rendering engine malfunction in Explorer.
	 * Notice that a borderwidth (or whatever) has been hardcoded into this!
	 */
	if ( Client.isExplorer && this.containingTabBoxBinding != null ) {
	
		var self = this;
		function fix () {
			var width = self.containingTabBoxBinding.getWidth ();
			if ( !isNaN ( width )) {
				width = width > 0 ? width - 1 : 0; // hardcode!
				self.bindingElement.style.width = new String ( width ) + "px";
			}
		}
		setTimeout ( fix, 250 );
		fix ();
	}
	
	DockTabsBinding.superclass.flex.call ( this );
}

/**
 * Handle crawler.
 * @implements {ICrawlerHandler}
 * @param {Crawler} crawler
 */
DockTabsBinding.prototype.handleCrawler = function ( crawler ) {
	
	DockTabsBinding.superclass.handleCrawler.call ( this, crawler );
	
	switch ( crawler.id ) {	
		case FlexBoxCrawler.ID :
			this._explorerFlexHack ();
			break;
	}
}

/*
 * This fixes a horrendous rendering engine malfunction in Explorer.
 * Notice that a borderwidth (or whatever) has been hardcoded into this!
 */
DockTabsBinding.prototype._explorerFlexHack = function () {
	
	if ( Client.isExplorer && this.containingTabBoxBinding != null ) {
	
		var self = this;
		function fix () {
			var width = self.containingTabBoxBinding.getWidth ();
			if ( !isNaN ( width )) {
				width = width > 0 ? width - 1 : 0; // hardcode!
				self.bindingElement.style.width = new String ( width ) + "px";
			}
		}
		setTimeout ( fix, 250 );
		fix ();
	}
}

/**
 * DockTabsBinding factory.
 * @param {DOMDocument} ownerDocument
 * @return {DockTabsBinding}
 */
DockTabsBinding.newInstance = function ( ownerDocument ) {

	var element = DOMUtil.createElementNS ( Constants.NS_UI, "ui:docktabs", ownerDocument );
	return UserInterface.registerBinding ( element, DockTabsBinding );
}