/*
 * The contents of this web application are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this web application except in compliance with 
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
 * for the specific language governing rights and limitations under the License.
 * 
 * The Original Code is owned by and the Initial Developer of the Original Code is 
 * Composite A/S (Danish business reg.no. 21744409). All Rights Reserved
 * 
 * Section 11 of the License is EXPRESSLY amended to include a provision stating 
 * that any dispute, including but not limited to disputes related to the enforcement 
 * of the License, to which Composite A/S as owner of the Original Code, as Initial 
 * Developer or in any other role, becomes a part to shall be governed by Danish law 
 * and be initiated before the Copenhagen City Court ("K�benhavns Byret")            
 */

ExplorerDecksBinding.prototype = new DecksBinding;
ExplorerDecksBinding.prototype.constructor = ExplorerDecksBinding;
ExplorerDecksBinding.superclass = DecksBinding.prototype;
ExplorerDecksBinding.NODENAME_DECK = "explorerdeck";

/**
 * @class
 */
function ExplorerDecksBinding () {

	/**
	 * @type {SystemLogger}
	 */
	this.logger = SystemLogger.getLogger ( "ExplorerDecksBinding" );
	
	/**
	 * Associating handles to decks.
	 * @type {HashMap<string><ExplorerDeckBinding>} 
	 */
	this._decks = {};
	
	/*
	 * Returnable.
	 */
	return this;
}

/**
 * @overloads {DecksBinding#onBindingAttach}
 */
ExplorerDecksBinding.prototype.onBindingAttach = function () {
	
	ExplorerDecksBinding.superclass.onBindingAttach.call ( this );
	this.addActionListener ( PageBinding.ACTION_ATTACHED );
	// this.dispatchAction ( Binding.ACTION_ATTACHED );
}

/**
 * Identifies binding.
 */
ExplorerDecksBinding.prototype.toString = function () {

	return "[ExplorerDecksBinding]";
}

/**
 * Building decks on startup.
 * @param {SystemViewDefinition} definition
 */
ExplorerDecksBinding.prototype.mountDefinition = function ( definition ) {

	if ( definition instanceof SystemViewDefinition ) {
	
		// construct ViewBinding
		var viewBinding = ViewBinding.newInstance ( this.bindingDocument );
		viewBinding.setType ( ViewBinding.TYPE_EXPLORERVIEW );
		viewBinding.setDefinition ( definition );
		
		// construct DeckBinding
		var deckBinding = ExplorerDeckBinding.newInstance ( this.bindingDocument );
		deckBinding.setAssociatedView ( viewBinding );
		this._decks [ definition.handle ] = deckBinding;
		
		// append
		deckBinding.add ( viewBinding );
		this.add ( deckBinding );
		
		//  attach - WebKit needs a short break here...
		function attach () {
			deckBinding.attach ();
			viewBinding.attach ();
		}
		if ( Client.isWebKit ) {
			setTimeout ( function () {
				attach ();
			}, 0 );
		} else {
			attach ();
		}
	}
}

/**
 * @param {string} handle
 */
ExplorerDecksBinding.prototype.setSelectionByHandle = function ( handle ) {
	
	var deckBinding = this._decks [ handle ];
	this.select ( deckBinding );
}

/**
 * Emulates recursive flex, but performs faster.
 * @param {int} pixels
 */
DecksBinding.prototype.expandBy = function ( pixels ) {
	
	var deck = this.getSelectedDeckBinding ();
	if ( deck ) {
		var height = this.bindingElement.offsetHeight + pixels;
		var view = deck.getAssociatedView ();
		this.bindingElement.style.height = height + "px";
		this.reflex ( true );
	}
}

/**
 * ExplorerDecksBinding factory.
 * @param {DOMDocument} ownerDocument
 * @return {ExplorerDecksBinding}
 */
ExplorerDecksBinding.newInstance = function ( ownerDocument ) {

	var element = DOMUtil.createElementNS ( Constants.NS_UI, "ui:explorerdecks", ownerDocument );
	return UserInterface.registerBinding ( element, ExplorerDecksBinding );
}